@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2007-2016 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2007-2018 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
     
     This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,7 @@
 #include "numbers.h"
 #include "mem.h"
 #include "error.h"
+#include "dpxconf.h"
 
 #include "dpxfile.h"
 
@@ -70,15 +71,8 @@
 #include "cmap_write.h"
 #include "fontmap.h"
 
-static int  verbose   = 0;
 static int  opt_flags = 0;
 
-void
-CIDFont_type0_set_verbose (void)
-{
-  verbose++;
-}
-
 void
 CIDFont_type0_set_flags (int flags)
 {
@@ -820,7 +814,7 @@ CIDFont_type0_dofont (CIDFont *font)
 
   CIDFontInfo_close(&info);
 
-  if (verbose > 1)
+  if (dpx_conf.verbose_level > 1)
     MESG("[%u/%u glyphs][%ld bytes]", num_glyphs, cs_count, destlen);
 
   CIDFont_type0_add_CIDSet(font, used_chars, last_cid);
@@ -890,12 +884,6 @@ CIDFont_type0_open (CIDFont *font, const char *name,
         DPXFCLOSE(fp);
       return -1;
     }
-
-    if (is_cid_font) {
-      cff_read_charsets(cffont);
-      opt->cff_charsets = cffont->charsets;
-      cffont->charsets = NULL;
-    }
   } else {
     if (!fp)
       return -1;
@@ -1270,7 +1258,7 @@ CIDFont_type0_t1cdofont (CIDFont *font)
 
   CIDFontInfo_close(&info);
 
-  if (verbose > 1)
+  if (dpx_conf.verbose_level > 1)
     MESG("[%u glyphs][%ld bytes]", num_glyphs, destlen);
 
   CIDFont_type0_add_CIDSet(font, used_chars, last_cid);
