@@ -41,11 +41,14 @@
 #include "error.h"
 #include "mfileio.h"
 #include "numbers.h"
+#include "dpxconf.h"
+#include "dpxutil.h"
 
 #include "pdfdev.h"
 #include "pdfdoc.h"
 #include "pdfparse.h"
-#include "pdfencrypt.h"
+#include "pdfresource.h"
+#include "pdfdraw.h"
 
 #include "fontmap.h"
 
@@ -144,6 +147,7 @@ static struct loaded_font
   spt_t size;
   int   source;     /* Source is either DVI or VF */
   uint32_t rgba_color;
+  int      xgs_id;  /* Transparency ExtGState */
   struct tt_longMetrics *hvmt;
   int   ascent;
   int   descent;
@@ -192,7 +196,6 @@ static struct font_def
 
 static int num_def_fonts = 0, max_def_fonts = 0;
 static int compute_boxes = 0, link_annot    = 1;
-static int verbose       = 0;
 
 #define DVI_PAGE_BUF_CHUNK              0x10000U        /* 64K should be plenty for most pages */
 
@@ -289,16 +292,6 @@ static int32_t get_buffered_unsigned_num(unsigned char num)
 
 #define skip_bufferd_bytes(n) dvi_page_buf_index += n
 
-void
-dvi_set_verbose (void)
-{
-  verbose++;
-  subfont_set_verbose();
-  tfm_set_verbose();
-  vf_set_verbose ();
-  spc_set_verbose();
-}
-
 unsigned int
 dvi_npages (void)
 {
@@ -323,7 +316,7 @@ check_id_bytes (void) {
 
 static void
 need_XeTeX (int c) {
-  if (!is_xdv)
+  if (dpx_conf.compat_mode != dpx_mode_xdv_mode)
     ERROR ("DVI opcode %i only valid for XeTeX", c);
 }
 
@@ -363,7 +356,8 @@ find_post (void)
   } 
 
   post_id_byte = ch;
-  is_xdv = (ch == XDV_ID || ch == XDV_ID_OLD);
+  if (ch == XDV_ID || ch == XDV_ID_OLD)
+    dpx_conf.compat_mode = dpx_mode_xdv_mode;
   is_ptex = ch == DVIV_ID;
 
   /* Make sure post_post is really there */
@@ -408,7 +402,7 @@ get_page_info (int32_t post_location)
   if (num_pages == 0) {
     ERROR("Page count is 0!");
   }
-  if (verbose > 2) {
+  if (dpx_conf.verbose_level > 2) {
     MESG("Page count:\t %4d\n", num_pages);
   }
 
@@ -461,7 +455,7 @@ get_dvi_info (int32_t post_location)
     ERROR("Capacity exceeded.");
   }
 
-  if (verbose > 2) {
+  if (dpx_conf.verbose_level > 2) {
     MESG("DVI File Info\n");
     MESG("Unit: %ld / %ld\n",    dvi_info.unit_num, dvi_info.unit_den);
     MESG("Magnification: %ld\n", dvi_info.mag);
@@ -490,7 +484,8 @@ get_preamble_dvi_info (void)
   }
 
   pre_id_byte = ch;
-  is_xdv = (ch == XDV_ID || ch == XDV_ID_OLD);
+  if (ch == XDV_ID || ch == XDV_ID_OLD)
+    dpx_conf.compat_mode = dpx_mode_xdv_mode;
   is_ptex = ch == DVI_ID; /* maybe */
   
   dvi_info.unit_num = get_positive_quad(dvi_file, "DVI", "unit_num");
@@ -504,13 +499,13 @@ get_preamble_dvi_info (void)
   }
   dvi_info.comment[ch] = '\0';
 
-  if (verbose > 2) {
+  if (dpx_conf.verbose_level > 2) {
     MESG("DVI File Info\n");
     MESG("Unit: %ld / %ld\n",    dvi_info.unit_num, dvi_info.unit_den);
     MESG("Magnification: %ld\n", dvi_info.mag);
   }
 
-  if (verbose) {
+  if (dpx_conf.verbose_level > 0) {
     MESG("DVI Comment: %s\n", dvi_info.comment);
   }
 
@@ -648,7 +643,7 @@ get_dvi_fonts (int32_t post_location)
       ERROR(invalid_signature);
     }
   }
-  if (verbose > 2) {
+  if (dpx_conf.verbose_level > 2) {
     unsigned  i;
 
     MESG("\n");
@@ -675,7 +670,7 @@ static void get_comment (void)
     ERROR(invalid_signature);
   }
   dvi_info.comment[length] = '\0';
-  if (verbose) {
+  if (dpx_conf.verbose_level > 0) {
     MESG("DVI Comment: %s\n", dvi_info.comment);
   }
 }
@@ -789,7 +784,7 @@ dvi_do_special (const void *buffer, int32_t size)
   mag    =  dvi_tell_mag();
 
   if (spc_exec_special(p, size, x_user, y_user, mag) < 0) {
-    if (verbose) {
+    if (dpx_conf.verbose_level > 0) {
       dump(p, p + size);
     }
   }
@@ -812,7 +807,7 @@ dvi_locate_font (const char *tfm_name, spt_t ptsize)
   int           subfont_id = -1, font_id; /* VF or device font ID */
   fontmap_rec  *mrec;
 
-  if (verbose)
+  if (dpx_conf.verbose_level > 0)
     MESG("<%s@%.2fpt", tfm_name, ptsize * dvi2pts);
 
   need_more_fonts(1);
@@ -858,7 +853,7 @@ dvi_locate_font (const char *tfm_name, spt_t ptsize)
     if (font_id >= 0) {
       loaded_fonts[cur_id].type    = VIRTUAL;
       loaded_fonts[cur_id].font_id = font_id;
-      if (verbose)
+      if (dpx_conf.verbose_level > 0)
         MESG("(VF)>");
       return  cur_id;
     }
@@ -888,7 +883,7 @@ dvi_locate_font (const char *tfm_name, spt_t ptsize)
       else {
         loaded_fonts[cur_id].type    = VIRTUAL;
         loaded_fonts[cur_id].font_id = font_id;
-        if (verbose)
+        if (dpx_conf.verbose_level > 0)
           MESG("(OVF)>");
         return  cur_id;
       }
@@ -937,7 +932,7 @@ dvi_locate_font (const char *tfm_name, spt_t ptsize)
   loaded_fonts[cur_id].type    = PHYSICAL;
   loaded_fonts[cur_id].font_id = font_id;
 
-  if (verbose)
+  if (dpx_conf.verbose_level > 0)
     MESG(">");
 
   return  cur_id;
@@ -963,7 +958,7 @@ dvi_locate_native_font (const char *filename, uint32_t index,
   struct tt_hhea_table *hhea;
   int is_dfont = 0, is_type1 = 0;
 
-  if (verbose)
+  if (dpx_conf.verbose_level > 0)
     MESG("<%s@%.2fpt", filename, ptsize * dvi2pts);
 
   if ((path = dpx_find_dfont_file(filename)) != NULL &&
@@ -1066,7 +1061,7 @@ dvi_locate_native_font (const char *filename, uint32_t index,
   loaded_fonts[cur_id].slant = mrec->opt.slant;
   loaded_fonts[cur_id].embolden = mrec->opt.bold;
 
-  if (verbose)
+  if (dpx_conf.verbose_level > 0)
     MESG(">");
 
   return cur_id;
@@ -1309,7 +1304,7 @@ dvi_rule (int32_t width, int32_t height)
 void
 dvi_dirchg (unsigned char dir)
 {
-  if (verbose)
+  if (dpx_conf.verbose_level > 0)
     fprintf(stderr, "  > dvi_dirchg %d\n", dir);
   dvi_state.d = dir;
   pdf_dev_set_dirmode(dvi_state.d); /* 0: horizontal, 1,3: vertical */
@@ -1490,6 +1485,20 @@ do_fnt (int32_t tex_id)
                                 def_fonts[i].point_size);
     }
     loaded_fonts[font_id].rgba_color = def_fonts[i].rgba_color;
+    /* Opacity: 0xff is fully opaque. */
+    if ((loaded_fonts[font_id].rgba_color & 0xff) == 0xff) {
+      loaded_fonts[font_id].xgs_id = -1;
+    } else {
+      pdf_obj *xgs_dict;
+      int      a = loaded_fonts[font_id].rgba_color & 0xff;
+
+      /* Inefficient but don't care as transparency is not expected to be frequently used. */
+      xgs_dict = pdf_new_dict();
+      pdf_add_dict(xgs_dict, pdf_new_name("Type"), pdf_new_name("ExtGState"));
+      pdf_add_dict(xgs_dict, pdf_new_name("ca"), pdf_new_number(a/255.0));
+      pdf_add_dict(xgs_dict, pdf_new_name("CA"), pdf_new_number(a/255.0));
+      loaded_fonts[font_id].xgs_id = pdf_defineresource("ExtGState", NULL, xgs_dict, 0);
+    }
     loaded_fonts[font_id].source = DVI;
     def_fonts[i].used    = 1;
     def_fonts[i].font_id = font_id;
@@ -1707,6 +1716,23 @@ do_glyphs (int do_actual_text)
       (double)((unsigned char)(font->rgba_color >> 16) & 0xff) / 255,
       (double)((unsigned char)(font->rgba_color >>  8) & 0xff) / 255);
     pdf_color_push(&color, &color);
+    /* Opacity:
+     * Enter graphics_mode and then enclose with save/resotre
+     * since pdf_color_pop() may not restore graphics state.
+     */
+    if (font->xgs_id >= 0) {
+      pdf_obj *ref;
+      char     resname[16];
+      char     content[22];
+
+      sprintf(resname, "Xtx_Gs_%08x", current_font);
+      ref = pdf_get_resource_reference(font->xgs_id);
+      pdf_doc_add_page_resource("ExtGState", resname, ref);
+      graphics_mode();
+      pdf_dev_gsave();
+      sprintf(content, " /%s gs ", resname);
+      pdf_doc_add_page_content(content, strlen(content));
+    }
   }
 
   for (i = 0; i < slen; i++) {
@@ -1755,6 +1781,10 @@ do_glyphs (int do_actual_text)
   }
 
   if (font->rgba_color != 0xffffffff) {
+    if (font->xgs_id >= 0) {
+      graphics_mode(); 
+      pdf_dev_grestore();
+    }
     pdf_color_pop();
   }
   RELEASE(xloc);
@@ -2222,18 +2252,75 @@ read_length (double *vp, double mag, const char **pp, const char *endptr)
   return  error;
 }
 
+static int
+scan_special_encrypt (int *key_bits, int32_t *permission, char *opassword, char *upassword,
+                      const char **curptr, const char *endptr)
+{
+  int   error = 0;
+  const char *p = *curptr;
+
+  skip_white(&p, endptr);
+
+  opassword[0] = '\0';
+  upassword[0] = '\0';
+  while (!error && p < endptr) {
+    char  *kp = parse_c_ident(&p, endptr);
+    if (!kp)
+      break;
+    else {
+      pdf_obj *obj;
+      skip_white(&p, endptr);
+      if (!strcmp(kp, "ownerpw")) {
+        if ((obj = parse_pdf_string(&p, endptr))) {
+          if (pdf_string_value(obj))
+            strncpy(opassword, pdf_string_value(obj), sizeof(opassword)-1);
+          pdf_release_obj(obj);
+        } else
+          error = -1;
+      } else if (!strcmp(kp, "userpw")) {
+        if ((obj = parse_pdf_string(&p, endptr))) {
+          if (pdf_string_value(obj))
+            strncpy(upassword, pdf_string_value(obj), sizeof(upassword)-1);
+          pdf_release_obj(obj);
+        } else
+          error = -1;
+      } else if (!strcmp(kp, "length")) {
+        if ((obj = parse_pdf_number(&p, endptr)) && PDF_OBJ_NUMBERTYPE(obj)) {
+          *key_bits = (unsigned) pdf_number_value(obj);
+        } else
+          error = -1;
+        if (obj)
+          pdf_release_obj(obj);
+      } else if (!strcmp(kp, "perm")) {
+        if ((obj = parse_pdf_number(&p, endptr)) && PDF_OBJ_NUMBERTYPE(obj)) {
+          *permission = (unsigned) pdf_number_value(obj);
+        } else
+          error = -1;
+        if (obj)
+          pdf_release_obj(obj);
+      } else
+        error = -1;
+      RELEASE(kp);
+    }
+    skip_white(&p, endptr);
+  }
+  *curptr = p;
+
+  return error;
+}
 
 static int
 scan_special (double *wd, double *ht, double *xo, double *yo, int *lm,
               int *majorversion, int *minorversion,
-              int *do_enc, int *key_bits, int32_t *permission,
-              char *owner_pw, char *user_pw,
+              int *enable_encryption, int *key_bits, int32_t *permission,
+              char *opassword, char *upassword,
               const char *buf, uint32_t size)
 {
   char  *q;
   const char *p = buf, *endptr;
   int    ns_pdf = 0, ns_dvipdfmx = 0, error = 0;
   double tmp;
+  double width = *wd, height = *ht; /* backup */
 
   endptr = p + size;
 
@@ -2354,50 +2441,9 @@ scan_special (double *wd, double *ht, double *xo, double *yo, int *lm,
         *majorversion = (int)strtol(kv, NULL, 10);
         RELEASE(kv);
       }
-    } else if (ns_pdf && !strcmp(q, "encrypt") && do_enc) {
-      *do_enc = 1;
-      *owner_pw = *user_pw = 0;
-      while (!error && p < endptr) {
-        char  *kp = parse_c_ident(&p, endptr);
-        if (!kp)
-          break;
-        else {
-          pdf_obj *obj;
-          skip_white(&p, endptr);
-          if (!strcmp(kp, "ownerpw")) {
-            if ((obj = parse_pdf_string(&p, endptr))) {
-              if (pdf_string_value(obj))
-                strncpy(owner_pw, pdf_string_value(obj), MAX_PWD_LEN);
-              pdf_release_obj(obj);
-            } else
-              error = -1;
-          } else if (!strcmp(kp, "userpw")) {
-            if ((obj = parse_pdf_string(&p, endptr))) {
-              if (pdf_string_value(obj))
-                strncpy(user_pw, pdf_string_value(obj), MAX_PWD_LEN);
-              pdf_release_obj(obj);
-            } else
-              error = -1;
-          } else if (!strcmp(kp, "length")) {
-            if ((obj = parse_pdf_number(&p, endptr)) && PDF_OBJ_NUMBERTYPE(obj)) {
-              *key_bits = (unsigned) pdf_number_value(obj);
-            } else
-              error = -1;
-            if (obj)
-              pdf_release_obj(obj);
-          } else if (!strcmp(kp, "perm")) {
-            if ((obj = parse_pdf_number(&p, endptr)) && PDF_OBJ_NUMBERTYPE(obj)) {
-              *permission = (unsigned) pdf_number_value(obj);
-            } else
-              error = -1;
-            if (obj)
-              pdf_release_obj(obj);
-          } else
-            error = -1;
-          RELEASE(kp);
-        }
-        skip_white(&p, endptr);
-      }
+    } else if (ns_pdf && !strcmp(q, "encrypt")) {
+      *enable_encryption = 1;
+      error = scan_special_encrypt(key_bits, permission, opassword, upassword, &p, endptr);
     } else if (ns_dvipdfmx && !strcmp(q, "config")) {
       read_config_special(&p, endptr);
     }
