@@ -2,7 +2,7 @@
 
     DVIPDFMx, an eXtended version of DVIPDFM by Mark A. Wicks.
 
-    Copyright (C) 2002-2015 by Jin-Hwan Cho, Matthias Franz, and Shunsaku Hirata,
+    Copyright (C) 2002-2018 by Jin-Hwan Cho, Matthias Franz, and Shunsaku Hirata,
     the DVIPDFMx project team.
     
     Copyright (c) 2006 SIL. (xdvipdfmx extensions for XeTeX support)
@@ -63,63 +63,67 @@
 
 #include "error.h"
 
-int is_xdv = 0;
-
-#ifdef XETEX
-const char *my_name = "xdvipdfmx";
+#if defined(LIBDPX)
+const char *my_name = "ApTeX";
 #else
-const char *my_name = "dvipdfmx";
-#endif
+const char *my_name;
+#endif /* LIBDPX */
 
-int compat_mode = 0;     /* 0 = dvipdfmx, 1 = dvipdfm */
-
-static int verbose = 0;
-
-static int mp_mode = 0;
-
-static long opt_flags = 0;
+static int    verbose   = 0;
 
+static int    opt_flags = 0;
 #define OPT_TPIC_TRANSPARENT_FILL (1 << 1)
 #define OPT_CIDFONT_FIXEDPITCH    (1 << 2)
 #define OPT_FONTMAP_FIRST_MATCH   (1 << 3)
 #define OPT_PDFDOC_NO_DEST_REMOVE (1 << 4)
+#define OPT_PDFOBJ_NO_PREDICTOR   (1 << 5)
+#define OPT_PDFOBJ_NO_OBJSTM      (1 << 6)
 
-static char   ignore_colors = 0;
-static double annot_grow    = 0.0;
-static int    bookmark_open = 0;
-static double mag           = 1.0;
-static int    font_dpi      = 600;
-static int    really_quiet  = 0;
+static int    pdf_version_major = 1;
+static int    pdf_version_minor = 5;
+static int    compression_level = 9;
+
+static char   ignore_colors    = 0;
+static double annot_grow       = 0.0;
+static int    bookmark_open    = 0;
+static double mag              = 1.0;
+static int    font_dpi         = 600;
+static int    enable_thumbnail = 0;
+static int    really_quiet     = 0;
 /*
  * Precision is essentially limited to 0.01pt.
  * See, dev_set_string() in pdfdev.c.
  */
-static int pdfdecimaldigits = 2;
+static int    pdfdecimaldigits  = 3;
 
 /* Image cache life in hours */
 /*  0 means erase all old images and leave new images */
 /* -1 means erase all old images and also erase new images */
 /* -2 means ignore image cache (default) */
-static int image_cache_life = -2;
+static int     image_cache_life = -2;
+/* Image format conversion filter template */
+static char   *filter_template  = NULL;
 
 /* Encryption */
-static int do_encryption    = 0;
-static unsigned key_bits    = 40;
-static unsigned permission  = 0x003C;
+static int     do_encryption = 0;
+static int     key_bits      = 40;
+static int32_t permission    = 0x003C;
 
 /* Page device */
 double paper_width  = 595.0;
 double paper_height = 842.0;
 static double x_offset = 72.0;
 static double y_offset = 72.0;
-char   landscape_mode    = 0;
+int    landscape_mode  = 0;
+static int    translate_origin = 0;
 
-int always_embed = 0; /* always embed fonts, regardless of licensing flags */
+/* Command line option takes precedence */
+static int     has_paper_option   = 0;
+static int     has_encrypt_option = 0;
 
 char *dvi_filename = NULL, *pdf_filename = NULL;
 
-static void
-read_config_file (const char *config);
+static void read_config_file (const char *config);
 
 static void
 set_default_pdf_filename(void)
@@ -127,7 +131,7 @@ set_default_pdf_filename(void)
   const char *dvi_base;
 
   dvi_base = xbasename(dvi_filename);
-  if (mp_mode &&
+  if (dpx_conf.compat_mode == dpx_mode_mpost_mode &&
       strlen(dvi_base) > 4 &&
       FILESTRCASEEQ(".mps", dvi_base + strlen(dvi_base) - 4)) {
     pdf_filename = NEW(strlen(dvi_base)+1, char);
@@ -153,12 +157,11 @@ show_version (void)
   printf ("This is %s Version " VERSION " by the DVIPDFMx project team,\n",
                    my_name);
   printf ("modified for TeX Live,\n");
-#ifdef XETEX
-  printf ("an extended version of DVIPDFMx, which in turn was\n");
-#endif
+  if (*my_name == 'x')
+    printf ("an extended version of DVIPDFMx, which in turn was\n");
   printf ("an extended version of dvipdfm-0.13.2c developed by Mark A. Wicks.\n");
-  printf ("\nCopyright (C) 2002-2015 the DVIPDFMx project team\n");
-  printf ("Copyright (C) 2006 SIL International.\n");
+  printf ("\nCopyright (C) 2002-2018 the DVIPDFMx project team\n");
+  printf ("Copyright (C) 2006-2018 SIL International.\n");
   printf ("\nThis is free software; you can redistribute it and/or modify\n");
   printf ("it under the terms of the GNU General Public License as published by\n");
   printf ("the Free Software Foundation; either version 2 of the License, or\n");
@@ -176,19 +179,20 @@ show_usage (void)
   printf ("\nOptions:\n"); 
   printf ("  -c \t\tIgnore color specials (for B&W printing)\n");
   printf ("  --dvipdfm\tEnable DVIPDFM emulation mode\n");
-  printf ("  -d number\tSet PDF decimal digits (0-5) [2]\n");
+  printf ("  -d number\tSet PDF decimal digits (0-5) [3]\n");
   printf ("  -f filename\tSet font map file name [pdftex.map]\n");
   printf ("  -g dimension\tAnnotation \"grow\" amount [0.0in]\n");
   printf ("  -h | --help \tShow this help message and exit\n");
   printf ("  -l \t\tLandscape mode\n");
   printf ("  -m number\tSet additional magnification [1.0]\n");
+  printf ("  --mvorigin\tTranslate the origin for MP inclusion\n");
   printf ("  -o filename\tSet output file name, \"-\" for stdout [DVIFILE.pdf]\n");
   printf ("  -p papersize\tSet papersize [a4]\n");
   printf ("  -q \t\tBe quiet\n");
   printf ("  -r resolution\tSet resolution (in DPI) for raster fonts [600]\n");
   printf ("  -s pages\tSelect page ranges [all pages]\n");
   printf ("  --showpaper\tShow available paper formats and exit\n");
-  printf ("  -t \t\tEmbed thumbnail images of PNG format [DVIFILE.1] \n");
+  printf ("  -t \t\tEmbed thumbnail images of PNG format (DVIFILE.pageno, pageno=int) \n");
   printf ("  --version\tOutput version information and exit\n");
   printf ("  -v \t\tBe verbose\n");
   printf ("  -vv\t\tBe more verbose\n");
@@ -204,6 +208,8 @@ show_usage (void)
   printf ("\t\t  0x0004 Treat all CIDFont as fixed-pitch font.\n");
   printf ("\t\t  0x0008 Do not replace duplicate fontmap entries.\n");
   printf ("\t\t  0x0010 Do not optimize PDF destinations.\n");
+  printf ("\t\t  0x0020 Do not use predictor filter for Flate compression.\n");
+  printf ("\t\t  0x0040 Do not use object stream.\n");
   printf ("\t\tPositive values are always ORed with previously given flags.\n");
   printf ("\t\tAnd negative values replace old values.\n");
   printf ("  -D template\tPS->PDF conversion command line template [none]\n");
@@ -217,7 +223,8 @@ show_usage (void)
   printf ("  -O number\tSet maximum depth of open bookmark items [0]\n");
   printf ("  -P number\tSet permission flags for PDF encryption [0x003C]\n");
   printf ("  -S \t\tEnable PDF encryption\n");
-  printf ("  -V number\tSet PDF minor version [%d]\n", PDF_VERSION_DEFAULT);
+  printf ("  -V number\tSet PDF version [%d.%d]\n",
+    PDF_VERSION_DEFAULT/10,PDF_VERSION_DEFAULT%10);
   printf ("\nAll dimensions entered on the command line are \"true\" TeX dimensions.\n");
   printf ("Argument of \"-s\" lists physical page ranges separated by commas,\n");
   printf ("\te.g., \"-s 1-3,5-6\".\n");
@@ -247,7 +254,11 @@ read_length (double *vp, const char **pp, const char *endptr)
 #define K_UNIT__CM  2
 #define K_UNIT__MM  3
 #define K_UNIT__BP  4
-    "pt", "in", "cm", "mm", "bp",
+#define K_UNIT__PC  5
+#define K_UNIT__DD  6
+#define K_UNIT__CC  7
+#define K_UNIT__SP  8
+    "pt", "in", "cm", "mm", "bp", "pc", "dd", "cc", "sp",
      NULL
   };
   int     k, error = 0;
@@ -282,6 +293,10 @@ read_length (double *vp, const char **pp, const char *endptr)
       case K_UNIT__CM: u *= 72.0 / 2.54 ; break;
       case K_UNIT__MM: u *= 72.0 / 25.4 ; break;
       case K_UNIT__BP: u *= 1.0 ; break;
+      case K_UNIT__PC: u *= 12.0 * 72.0 / 72.27 ; break;
+      case K_UNIT__DD: u *= 1238.0 / 1157.0 * 72.0 / 72.27 ; break;
+      case K_UNIT__CC: u *= 12.0 * 1238.0 / 1157.0 * 72.0 / 72.27 ; break;
+      case K_UNIT__SP: u *= 72.0 / (72.27 * 65536) ; break;
       default:
         WARN("Unknown unit of measure: %s", q);
         error = -1;
@@ -325,7 +340,7 @@ select_paper (const char *paperspec)
 
 struct page_range 
 {
-  long first, last;
+  int first, last;
 } *page_ranges = NULL;
 
 int num_page_ranges = 0;
@@ -384,281 +399,319 @@ select_pages (const char *pagespec)
   return;
 }
 
-#define POP_ARG() {argv += 1; argc -= 1;}
-/* It doesn't work as expected (due to dvi filename). */
-#define CHECK_ARG(n,m) if (argc < (n) + 1) {\
-  fprintf (stderr, "\nMissing %s after \"-%c\".\n", (m), *flag);\
-  usage();\
-}
-
-static void
-set_verbose (int argc, char *argv[])
+/* Ask passwords for encryption */
+#ifdef WIN32
+/* Broken on mintty? */
+#include <conio.h>
+#define getch _getch
+static char *
+getpass (const char *prompt)
 {
-  while (argc > 0) {
-    if(*argv[0] == '-') {
-      char *flag;
+  static char pwd_buf[MAX_PWD_LEN];
+  size_t i;
 
-      for (flag = argv[0] + 1; *flag != 0; flag++) {
-        if (*flag == '-')
-          break;
-        if (*flag == 'q')
-          really_quiet = 2;
-        if (*flag == 'v')
-          verbose++;
-      }
-    }
-    POP_ARG();
+  fputs(prompt, stderr);
+  fflush(stderr);
+  for (i = 0; i < sizeof(pwd_buf)-1; i++) {
+    pwd_buf[i] = getch();
+    if (pwd_buf[i] == '\r' || pwd_buf[i] == '\n')
+      break;
+    fputs("*", stderr);
+    fflush(stderr);
+  }
+  pwd_buf[i] = '\0';
+  fputs("\n", stderr);
+  return pwd_buf;
+}
+#else  /* !WIN32 */
+#include <unistd.h>
+#endif /* WIN32 */
+
+static int
+get_enc_password (char *oplain, char *uplain)
+{
+  char  *retry_passwd;
+
+  while (1) {
+    strncpy(oplain, getpass("Owner password: "), MAX_PWD_LEN);
+    retry_passwd = getpass("Re-enter owner password: ");
+    if (!strncmp(oplain, retry_passwd, MAX_PWD_LEN))
+      break;
+    fputs("Password is not identical.\nTry again.\n", stderr);
+    fflush(stderr);
+  }
+  while (1) {
+    strncpy(uplain, getpass("User password: "), MAX_PWD_LEN);
+    retry_passwd = getpass("Re-enter user password: ");
+    if (!strncmp(uplain, retry_passwd, MAX_PWD_LEN))
+      break;
+    fputs("Password is not identical.\nTry again.\n", stderr);
+    fflush(stderr);
   }
 
-  if (!really_quiet) {
+  return 0;
+}
+
+static const char *optstrig = ":hD:r:m:g:x:y:o:s:p:clf:i:qtvV:z:d:I:K:P:O:MSC:Ee";
+
+static struct option long_options[] = {
+  {"help", 0, 0, 'h'},
+  {"version", 0, 0, 130},
+  {"showpaper", 0, 0, 131},
+  {"dvipdfm", 0, 0, 132},
+  {"mvorigin", 0, 0, 1000},
+  {"kpathsea-debug", 1, 0, 133},
+  {0, 0, 0, 0}
+};
+
+static void
+do_args_first_pass (int argc, char *argv[], const char *source, int unsafe)
+{
+  int c;
+
+  optind = 1;
+  while ((c = getopt_long(argc, argv, optstrig, long_options, NULL)) != -1) {
+    switch(c) {
+    case 'h':
+      show_usage();
+      exit(0);
+      break;
+
+    case 130: /* --version */
+      show_version();
+      exit(0);
+      break;
+
+    case 131: /* --showpaper */
+      dumppaperinfo();
+      exit(0);
+      break;
+
+    case 132: /* --dvipdfm */
+      dpx_conf.compat_mode = dpx_mode_compat_mode;
+      break;
+
+    case 133: /* --kpathsea-debug */
+      kpathsea_debug = atoi(optarg);
+      break;
+
+    case 1000: /* --mvorigin */
+      translate_origin = 1;
+      break;
+
+    case 'q':
+      really_quiet = 2;
+      break;
+
+    case 'v':
+      verbose++;
+      break;
+
+    case 'M':
+      dpx_conf.compat_mode = dpx_mode_mpost_mode;
+      break;
+
+    default: /* ignore everything else */
+      break;
+    }
+  }
+
+  if (really_quiet)
+    shut_up(really_quiet);
+  else {
     int i;
 
     for (i = 0; i < verbose; i++) {
-      dvi_set_verbose();
-      pdf_dev_set_verbose();
-      pdf_doc_set_verbose();
-      pdf_enc_set_verbose();
-      pdf_obj_set_verbose();
-      pdf_fontmap_set_verbose();
-      dpx_file_set_verbose();
-      tt_aux_set_verbose();
+      dpx_conf.verbose_level++;
     }
   }
+
+  if (argc > optind + 1) {
+    fprintf(stderr, "%s: Multiple dvi filenames?", my_name);
+    usage();
+  } else if (argc > optind) { /* last argument */
+    dvi_filename = NEW(strlen(argv[optind]) + 5, char);  /* space to append ".dvi" */
+    strcpy(dvi_filename, argv[optind]);
+  }
 }
 
-
+/* Set "unsafe" to non-zero value when parsing config specials to
+ * disallow overriding "D" option value.
+ */
 static void
-do_args (int argc, char *argv[])
+do_args_second_pass (int argc, char *argv[], const char *source, int unsafe)
 {
-  while (argc > 0 && *argv[0] == '-') {
-    char *flag, *nextptr;
-    const char *nnextptr;
+  int         c;
+  char       *nextptr;
+  const char *nnextptr;
 
-    for (flag = argv[0] + 1; *flag != 0; flag++) {
-      switch (*flag) {
-      case '-':
-        if (flag == argv[0] + 1 && ++flag) {
-          if (!strcmp(flag, "help")) {
-            show_usage();
-            exit(0);
-          } else if (!strcmp(flag, "version")) {
-            show_version();
-            exit(0);
-          } else if (!strcmp(flag, "dvipdfm")) {
-            compat_mode = 1;
-            goto Out_of_For_Loop;
-          } else if (!strcmp(flag, "kpathsea-debug")) {
-            int value;
-            CHECK_ARG(1, "kpathsearch debugging flags");
-            value = atoi(argv[1]);
-            if (value < 0)
-              ERROR("Invalid kpathsearch debugging flags specified: %s", argv[1]);
-            kpathsea_debug = value;
-            POP_ARG();
-            goto Out_of_For_Loop;
-          }
-        }
-        fprintf(stderr, "Unknown option \"%s\"", argv[0]);
-        usage();
-      case 'D':
-        CHECK_ARG(1, "PS->PDF conversion command line template");
-        set_distiller_template(argv[1]);
-        POP_ARG();
-        break;
-      case 'r':
-        CHECK_ARG(1, "bitmap font dpi");
-        font_dpi = atoi(argv[1]);
-        if (font_dpi <= 0)
-          ERROR("Invalid bitmap font dpi specified: %s", argv[1]);
-        POP_ARG();
-        break;
-      case 'm':
-        CHECK_ARG(1, "magnification value");
-        mag = strtod(argv[1], &nextptr);
-        if (mag < 0.0 || nextptr == argv[1])
-          ERROR("Invalid magnification specifiied: %s", argv[1]);
-        POP_ARG();
-        break;
-      case 'g':
-        CHECK_ARG(1, "annotation \"grow\" amount");
-        nnextptr = nextptr = argv[1];
-        read_length(&annot_grow, &nnextptr, nextptr + strlen(nextptr));
-        POP_ARG();
-        break;
-      case 'x':
-        CHECK_ARG(1, "horizontal offset value");
-        nnextptr = nextptr = argv[1];
-        read_length(&x_offset, &nnextptr, nextptr + strlen(nextptr));
-        POP_ARG();
-        break;
-      case 'y':
-        CHECK_ARG(1, "vertical offset value");
-        nnextptr = nextptr = argv[1];
-        read_length(&y_offset, &nnextptr, nextptr + strlen(nextptr));
-        POP_ARG();
-        break;
-      case 'o':
-        CHECK_ARG(1, "output file name");
-        pdf_filename = NEW (strlen(argv[1])+1,char);
-        strcpy(pdf_filename, argv[1]);
-        POP_ARG();
-        break;
-      case 's':
-        CHECK_ARG(1, "page selection specification");
-        select_pages(argv[1]);
-        POP_ARG();
-        break;
-      case 't':
-        pdf_doc_enable_manual_thumbnails();
-        break;
-      case 'p':
-        CHECK_ARG(1, "paper format/size");
-        select_paper(argv[1]);
-        POP_ARG();
-        break;
-      case 'c':
-        ignore_colors = 1;
-        break;
-      case 'l':
-        landscape_mode = 1;
-        break;
-      case 'f':
-        CHECK_ARG(1, "fontmap file name");
-        if (opt_flags & OPT_FONTMAP_FIRST_MATCH)
-          pdf_load_fontmap_file(argv[1], FONTMAP_RMODE_APPEND);
-        else
-          pdf_load_fontmap_file(argv[1], FONTMAP_RMODE_REPLACE);
-        POP_ARG();
-        break;
-      case 'i':
-        CHECK_ARG(1, "subsidiary config file");
-        read_config_file(argv[1]);
-        POP_ARG();
-        break;
-      case 'q': case 'v':
-        break;
-      case 'V':
-      {
-        int ver_minor;
+  optind = 1;
+  while ((c = getopt_long(argc, argv, optstrig, long_options, NULL)) != -1) {
+    switch(c) {
+    case 'h': case 130: case 131: case 132: case 133: case 1000: case 'q': case 'v': case 'M': /* already done */
+      break;
 
-        if (isdigit((unsigned char)*(flag+1))) {
-          flag++;
-          ver_minor = atoi(flag);
-        } else {
-          CHECK_ARG(1, "PDF minor version number");
-          ver_minor = atoi(argv[1]);
-          POP_ARG();
-        }
-        if (ver_minor < PDF_VERSION_MIN) {
-          WARN("PDF version 1.%d not supported. Using PDF 1.%d instead.",
-               ver_minor, PDF_VERSION_MIN);
-          ver_minor = PDF_VERSION_MIN;
-        } else if (ver_minor > PDF_VERSION_MAX) {
-          WARN("PDF version 1.%d not supported. Using PDF 1.%d instead.",
-               ver_minor, PDF_VERSION_MAX);
-          ver_minor = PDF_VERSION_MAX;
-        }
-        pdf_set_version((unsigned) ver_minor);
+    case 'D':
+      if (unsafe) {
+        WARN("Ignoring \"D\" option for dvipdfmx:config special. (unsafe)");
+      } else {
+        if (filter_template)
+          RELEASE(filter_template);
+        filter_template = NEW(strlen(optarg)+1, char);
+        strcpy(filter_template, optarg);
       }
       break;
-      case 'z':
-      {
-        int level;
 
-        if (isdigit((unsigned char)*(flag+1))) {
-          flag++;
-          level = atoi(flag);
-        } else {
-          CHECK_ARG(1, "compression level");
-          level = atoi(argv[1]);
-          POP_ARG();
-        }
-        pdf_set_compression(level);
-      }
+    case 'r':
+      if ((font_dpi = atoi(optarg)) <= 0)
+        ERROR("Invalid bitmap font dpi specified: %s", optarg);
       break;
-      case 'd':
-        if (isdigit((unsigned char)*(flag+1))) {
-          flag++;
-          pdfdecimaldigits = atoi(flag);
-        } else {
-          CHECK_ARG(1, "number of fractional digits");
-          pdfdecimaldigits = atoi(argv[1]);
-          POP_ARG();
-        }
-        break;
-      case 'I':
-        CHECK_ARG(1, "image cache life in hours");
-        image_cache_life = atoi(argv[1]);
-        POP_ARG();
-        break;
-      case 'S':
-        do_encryption = 1;
-        break;
-      case 'K': 
-        CHECK_ARG(1, "encryption key length");
-        key_bits = (unsigned) atoi(argv[1]);
-        if (key_bits < 40 || key_bits > 128 || (key_bits & 0x7))
-          ERROR("Invalid encryption key length specified: %s", argv[1]);
-        POP_ARG();
-        break;
-      case 'P': 
-        CHECK_ARG(1, "encryption permission flag");
-        permission = (unsigned) strtoul(argv[1], &nextptr, 0);
-        if (nextptr == argv[1])
-          ERROR("Invalid encryption permission flag: %s", argv[1]);
-        POP_ARG();
-        break;
-      case 'O':
-        /* Bookmark open level */
-        CHECK_ARG(1, "bookmark open level");
-        bookmark_open = atoi(argv[1]);
-        POP_ARG();
-        break;
-      case 'M':
-        mp_mode = 1;
-        break;
-      case 'C':
-        CHECK_ARG(1, "a number");
-        {
-          long flags;
 
-          flags = (unsigned) strtol(argv[1], &nextptr, 0);
-          if (nextptr == argv[1])
-            ERROR("Invalid flag: %s", argv[1]);
-          if (flags < 0)
-            opt_flags  = -flags;
-          else
-            opt_flags |=  flags;
-        }
-        POP_ARG();
-        break;
-      case 'E':
-        always_embed = 1;
+    case 'm':
+      if ((mag = strtod(optarg, &nextptr)) < 0.0 || nextptr == optarg)
+        ERROR("Invalid magnification specified: %s", optarg);
+      break;
+
+    case 'g':
+      nnextptr = nextptr = optarg;
+      read_length(&annot_grow, &nnextptr, nextptr + strlen(nextptr));
+      break;
+
+    case 'x':
+      nnextptr = nextptr = optarg;
+      read_length(&x_offset, &nnextptr, nextptr + strlen(nextptr));
+      break;
+
+    case 'y':
+      nnextptr = nextptr = optarg;
+      read_length(&y_offset, &nnextptr, nextptr + strlen(nextptr));
+      break;
+
+    case 'o':
+      pdf_filename = NEW (strlen(optarg)+1, char);
+      strcpy(pdf_filename, optarg);
+      break;
+
+    case 's':
+      select_pages(optarg);
+      break;
+
+    case 't':
+      enable_thumbnail = 1;
+      break;
+
+    case 'p':
+      select_paper(optarg);
+      has_paper_option = 1;
+      break;
+
+    case 'c':
+      ignore_colors = 1;
+      break;
+
+    case 'l':
+      landscape_mode = 1;
+      break;
+
+    case 'f':
+      if (opt_flags & OPT_FONTMAP_FIRST_MATCH)
+        pdf_load_fontmap_file(optarg, FONTMAP_RMODE_APPEND);
+      else
+        pdf_load_fontmap_file(optarg, FONTMAP_RMODE_REPLACE);
+      break;
+
+    case 'i':
+    {
+      int optind_save= optind;
+      read_config_file(optarg);
+      optind = optind_save;
+      break;
+    }
+
+    case 'V':
+    {
+      if (strchr(optarg, '.')) { /* new format X.X */
+        double tmp = atof(optarg);
+        pdf_version_major = (int) tmp;
+        pdf_version_minor = (int) (10 * (tmp - pdf_version_major) + 0.5);
+      } else {
+        pdf_version_major = 1;
+        pdf_version_minor = atoi(optarg);
+      }
+      break;
+    }
+
+    case 'z':
+      compression_level = atoi(optarg);
+      break;
+
+    case 'd':
+      pdfdecimaldigits = atoi(optarg);
+      break;
+
+    case 'I':
+      image_cache_life = atoi(optarg);
+      break;
+
+    case 'S':
+      do_encryption = 1;
+      has_encrypt_option = 1;
+      break;
+
+    case 'K':
+      key_bits = (unsigned) atoi(optarg);
+      if (!(key_bits >= 40 && key_bits <= 128 && (key_bits % 8 == 0)) &&
+            key_bits != 256)
+        ERROR("Invalid encryption key length specified: %s", optarg);
+      break;
+
+    case 'P':
+      permission = (unsigned) strtoul(optarg, &nextptr, 0);
+      if (nextptr == optarg)
+        ERROR("Invalid encryption permission flag: %s", optarg);
+      break;
+
+    case 'O':
+      bookmark_open = atoi(optarg);
+      break;
+
+    case 'C':
+    {
+      int flags = (unsigned) strtol(optarg, &nextptr, 0);
+      if (nextptr == optarg)
+        ERROR("Invalid flag: %s", optarg);
+      if (flags < 0)
+        opt_flags  = -flags;
+      else
+        opt_flags |=  flags;
+      break;
+    }
+
+    case 'E':
+      dpx_conf.ignore_font_license = 1;
+      break;
+
+    case 'e':
+      if (dpx_conf.compat_mode == dpx_mode_compat_mode) {
+        WARN("dvipdfm \"-e\" option not supported.");
         break;
-      case 'h':
-        show_usage();
-        exit(0);
-      case 'e':
-        if (compat_mode) {
-          WARN("dvipdfm \"-e\" option not supported.");
-          break;
-        } /* else fall through */
-      default:
-        fprintf (stderr, "Unknown option in \"%s\"", argv[0]);
-        usage();
-      }
+      } /* else fall through */
+
+    default:
+      fprintf(stderr, "%s: %s \"-%c\"\n", source ? source : my_name,
+              c == ':' ? "Missing argument for" : "Unknown option",
+              optopt); 
+      usage();
     }
-  Out_of_For_Loop:
-    POP_ARG();
   }
 
-  if (argc > 1) {
-    fprintf(stderr, "Multiple dvi filenames?");
-    usage();
-  } else if (argc > 0) {
-    dvi_filename = NEW(strlen(argv[0]) + 5, char);  /* space to append ".dvi" */
-    strcpy(dvi_filename, argv[0]);
+  if (source) {
+    if (argc > optind)
+      fprintf(stderr, "%s: Unexpected argument in \"%s %s\".\n", source, argv[1], argv[2]);
+    return;
   }
+
+  return;
 }
 
 static void
@@ -670,6 +723,8 @@ cleanup (void)
     RELEASE(pdf_filename);
   if (page_ranges)
     RELEASE(page_ranges);
+  if (filter_template)
+    RELEASE(filter_template);
 }
 
 static void
@@ -678,17 +733,18 @@ read_config_file (const char *config)
   const char *start, *end;
   char *option;
   FILE *fp;
+  static char argv0[] = "config_file";
+  char *argv[3];
 
   fp = DPXFOPEN(config, DPX_RES_TYPE_TEXT);
   if (!fp) {
     WARN("Could not open config file \"%s\".", config);
     return;
   }
+  argv[0] = argv0;
   while ((start = mfgets (work_buffer, WORK_BUFFER_SIZE, fp)) != NULL) {
-    char *argv[2];
-    int   argc;
+    int   argc = 1;
 
-    argc = 0;
     end = work_buffer + strlen(work_buffer);
     skip_white (&start, end);
     if (start >= end)
@@ -696,23 +752,23 @@ read_config_file (const char *config)
     /* Build up an argument list as if it were passed on the command
        line */
     if ((option = parse_ident (&start, end))) {
-      argc = 1;
-      argv[0] = NEW (strlen(option)+2, char);
-      strcpy (argv[0]+1, option);
+      argc = 2;
+      argv[1] = NEW (strlen(option)+2, char);
+      strcpy (argv[1]+1, option);
       RELEASE (option);
-      *argv[0] = '-';
+      *argv[1] = '-';
       skip_white (&start, end);
       if (start < end) {
         argc += 1;
         if (*start == '"') {
-          argv[1] = parse_c_string (&start, end);
+          argv[2] = parse_c_string (&start, end);
         }
         else
-          argv[1] = parse_ident (&start, end);
+          argv[2] = parse_ident (&start, end);
       }
     }
-    do_args (argc, argv);
-    while (argc > 0) {
+    do_args_second_pass (argc, argv, config, 0);
+    while (argc > 1) {
       RELEASE (argv[--argc]);
     }
   }
@@ -720,6 +776,43 @@ read_config_file (const char *config)
     MFCLOSE(fp);
 }
 
+void
+read_config_special (const char **start, const char *end)
+{
+  char *option;
+  static char argv0[] = "config_special";
+  char *argv[3];
+  int argc = 1;
+
+  argv[0] = argv0;
+
+  skip_white (start, end);
+  if (*start >= end)
+    return;
+  /* Build up an argument list as if it were passed on the command
+     line */
+  if ((option = parse_ident (start, end))) {
+    argc = 2;
+    argv[1] = NEW (strlen(option)+2, char);
+    strcpy (argv[1]+1, option);
+    RELEASE (option);
+    *argv[1] = '-';
+    skip_white (start, end);
+    if (*start < end) {
+      argc += 1;
+      if (**start == '"') {
+        argv[2] = parse_c_string (start, end);
+      }
+      else
+      argv[2] = parse_ident (start, end);
+    }
+  }
+  do_args_second_pass (argc, argv, argv0, 1); /* Set to unsafe */
+  while (argc > 1) {
+    RELEASE (argv[--argc]);
+  }
+}
+
 static void
 system_default (void)
 {
@@ -736,8 +829,12 @@ error_cleanup (void)
   pdf_close_images();  /* delete temporary files */
   pdf_error_cleanup();
   if (pdf_filename) {
-    remove(pdf_filename);
-    fprintf(stderr, "\nOutput file removed.\n");
+    if (pdf_get_output_file()) {
+      remove(pdf_filename);
+      fprintf(stderr, "\nOutput file removed.\n");
+    } else {
+      fprintf(stderr, "\nNo output PDF file written.\n");
+    }
   }
 }
 
@@ -787,15 +884,16 @@ do_dvi_pages (void)
     while (dvi_npages()) {
       if (page_no < dvi_npages()) {
         double w, h, xo, yo;
-        char   lm;
+        int    lm;
 
         MESG("[%d", page_no+1);
         /* Users want to change page size even after page is started! */
         page_width = paper_width; page_height = paper_height;
         w = page_width; h = page_height; lm = landscape_mode;
         xo = x_offset; yo = y_offset;
-        dvi_scan_specials(page_no, &w, &h, &xo, &yo, &lm, NULL, NULL, NULL, NULL, NULL, NULL);
-        if (lm != landscape_mode) {
+        dvi_scan_specials(page_no,
+                          &w, &h, &xo, &yo, &lm, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+        if (lm != landscape_mode) { /* already swapped for the first page */
           SWAP(w, h);
           landscape_mode = lm;
         }
@@ -850,7 +948,7 @@ do_mps_pages (void)
     mps_do_page(fp);
     MFCLOSE(fp);
   } else {
-    long  i, page_no, step, page_count = 0;
+    int  i, page_no, step, page_count = 0;
     char *filename;
     /* Process filename.1, filename.2,... */
     filename = NEW(strlen(dvi_filename) + 16 + 1, char);
@@ -861,7 +959,7 @@ do_mps_pages (void)
         if (page_no < 0)
           ERROR("Invalid page number for MPS input: %d", page_no);
 
-        sprintf(filename, "%s.%ld", dvi_filename, page_no + 1);
+        sprintf(filename, "%s.%d", dvi_filename, page_no + 1);
         fp = MFOPEN(filename, FOPEN_RBIN_MODE);
         if (fp) {
           MESG("[%ld<%s>", page_no + 1, filename);
@@ -887,18 +985,16 @@ do_mps_pages (void)
   }
 }
 
-#if defined(XETEX)
-/* At present no DLL for xdvipdfmx */
-#undef DLLPROC
-#else
+#if !defined(LIBDPX)
 /* Support to make DLL in W32TeX */
 #define DLLPROC dlldvipdfmxmain
-#endif
+
 #if defined(WIN32) && !defined(__MINGW32__) && !defined(MIKTEX) && defined(DLLPROC)
 extern __declspec(dllexport) int DLLPROC (int argc, char *argv[]);
 #else
 #undef DLLPROC
 #endif
+#endif /* !LIBDPX */
 
 #if defined(MIKTEX)
 #  define main Main
@@ -908,11 +1004,19 @@ int
 #if defined(DLLPROC)
 DLLPROC (int argc, char *argv[])
 #else
-CDECL main (int argc, char *argv[])
+#if defined(LIBDPX)
+dvipdfmx_main (int argc, char *argv[])
+#else
+main (int argc, char *argv[])
+#endif /* LIBDPX */
 #endif
 {
-  double dvi2pts;
-  char *base;
+  double             dvi2pts;
+  char              *base;
+  struct pdf_setting settings;
+  char               uplain[MAX_PWD_LEN+1], oplain[MAX_PWD_LEN+1]; /* encryption password */
+  const char        *ids[4] = {NULL, NULL, NULL, NULL};
+
 #ifdef WIN32
   int ac;
   char **av, *enc;
@@ -936,6 +1040,7 @@ CDECL main (int argc, char *argv[])
                (STREQ (argv[1], "--xbb") ||
                 STREQ (argv[1], "--extractbb") ||
                 STREQ (argv[1], "--dvipdfm") ||
+                STREQ (argv[1], "--dvipdfmx") ||
                 STREQ (argv[1], "--ebb"))) {
     argc--;
     base = argv++[1]+2;
@@ -948,45 +1053,32 @@ CDECL main (int argc, char *argv[])
   }
   if (FILESTRCASEEQ (base, "ebb")) {
     my_name = "ebb";
-    compat_mode = 1;
+    dpx_conf.compat_mode = dpx_mode_compat_mode;
     return extractbb (argc, argv);
   }
-
-  /* Special-case single option --help, --showpaper, or --version, to avoid
-     possible diagnostics about config files, etc.  */
-  if (argc == 2 && STREQ (argv[1], "--help")) {
-    show_usage();
-    exit(0);
-  } else if (argc == 2 && STREQ (argv[1], "--version")) {
-    show_version();
-    exit(0);
-  } else if (argc == 2 && STREQ (argv[1], "--showpaper")) {
-    dumppaperinfo();
-    exit(0);
-  }
-
   if (FILESTRCASEEQ (base, "dvipdfm"))
-    compat_mode = 1;
+    dpx_conf.compat_mode = dpx_mode_compat_mode;
+  if (FILESTRCASEEQ (base, "dvipdfmx"))
+    my_name = "dvipdfmx";
   else
-    free (base);
+    my_name = "xdvipdfmx";
+  
+  opterr = 0;
+
+  /* Special-case single option --mvorigin, --help, --showpaper, or --version,
+   * to avoid possible diagnostics about config files, etc.
+   * Also handle -q and -v that cannot be set in config file.
+   * Get input DVI file name too.
+   */
+  do_args_first_pass(argc, argv, NULL, 0);
 
   paperinit();
   system_default();
 
-  argv+=1;
-  argc-=1;
-
-  set_verbose(argc, argv);
-  /* quiet mode cannot be set in config file */
-  if (really_quiet)
-    shut_up(really_quiet);
-
   pdf_init_fontmaps(); /* This must come before parsing options... */
 
   read_config_file(DPX_CONFIG_FILE);
 
-  do_args (argc, argv);
-
 #ifndef MIKTEX
   kpse_init_prog("", font_dpi, NULL, NULL);
   kpse_set_program_enabled(kpse_pk_format, true, kpse_src_texmf_cnf);
@@ -1011,91 +1103,114 @@ CDECL main (int argc, char *argv[])
   MESG("%s -> %s\n", dvi_filename ? dvi_filename : "stdin",
                      pdf_filename ? pdf_filename : "stdout");
 
-  pdf_enc_compute_id_string(dvi_filename, pdf_filename);
-  if (do_encryption) {
-    if (key_bits > 40 && pdf_get_version() < 4)
-      ERROR("Chosen key length requires at least PDF 1.4. "
-            "Use \"-V 4\" to change.");
-    pdf_enc_set_passwd(key_bits, permission, NULL, NULL);
-  }
-
-  if (mp_mode) {
+  if (dpx_conf.compat_mode == dpx_mode_mpost_mode) {
     x_offset = 0.0;
     y_offset = 0.0;
     dvi2pts  = 0.01; /* dvi2pts controls accuracy. */
   } else {
-    unsigned ver_minor = 0;
-    char owner_pw[MAX_PWD_LEN], user_pw[MAX_PWD_LEN];
     /* Dependency between DVI and PDF side is rather complicated... */
     dvi2pts = dvi_init(dvi_filename, mag);
     if (dvi2pts == 0.0)
       ERROR("dvi_init() failed!");
+    ids[3] = dvi_comment(); /* Set PDF Creator entry */
+    dvi_scan_specials(0,
+                      &paper_width, &paper_height,
+                      &x_offset, &y_offset, &landscape_mode,
+                      &pdf_version_major, &pdf_version_minor,
+                      &do_encryption, &key_bits, &permission, oplain, uplain);
+  }
 
-    pdf_doc_set_creator(dvi_comment());
+  /* Command-line options take precedence */
+  {
+    int has_encrypt_special = do_encryption;
 
-    if (do_encryption)
-      /* command line takes precedence */
-      dvi_scan_specials(0, &paper_width, &paper_height, &x_offset, &y_offset, &landscape_mode,
-			&ver_minor, NULL, NULL, NULL, NULL, NULL);
-    else {
-      dvi_scan_specials(0, &paper_width, &paper_height, &x_offset, &y_offset, &landscape_mode,
-			&ver_minor, &do_encryption, &key_bits, &permission, owner_pw, user_pw);
-      if (do_encryption) {
-	if (key_bits < 40 || key_bits > 128 || (key_bits & 0x7))
-	  ERROR("Invalid encryption key length specified: %u", key_bits);
-	else if (key_bits > 40 && pdf_get_version() < 4)
-	  ERROR("Chosen key length requires at least PDF 1.4. "
-		"Use \"-V 4\" to change.");
-	do_encryption = 1;
-	pdf_enc_set_passwd(key_bits, permission, owner_pw, user_pw);
-      }
-    }
-    if (ver_minor >= PDF_VERSION_MIN && ver_minor <= PDF_VERSION_MAX) {
-      pdf_set_version(ver_minor);
-    }
-    if (landscape_mode) {
-      SWAP(paper_width, paper_height);
+    do_args_second_pass(argc, argv, NULL, 0);
+    if (do_encryption && !has_encrypt_special) {
+      get_enc_password(oplain, uplain);
     }
   }
 
-  pdf_files_init();
+  /* Encryption and Other Settings */
+  {
+#define PRODUCER \
+"%s-%s, Copyright 2002-2015 by Jin-Hwan Cho, Matthias Franz, and Shunsaku Hirata"
+    char producer[256];
 
+    memset(&settings.encrypt, 0, sizeof(struct pdf_enc_setting));
+    settings.enable_encrypt = do_encryption;
+    sprintf(producer, PRODUCER, my_name, VERSION);
+    settings.encrypt.use_aes          = 1;
+    settings.encrypt.encrypt_metadata = 1;
+    settings.encrypt.key_size   = key_bits;
+    settings.encrypt.permission = permission;
+    settings.encrypt.uplain     = uplain;
+    settings.encrypt.oplain     = oplain;
+    ids[0] = producer;
+    ids[1] = dvi_filename;
+    ids[2] = pdf_filename;
+  }
+
+  if (opt_flags & OPT_PDFOBJ_NO_OBJSTM) {
+    settings.object.enable_objstm = 0;
+  } else {
+    settings.object.enable_objstm = 1;
+  }
+  if (opt_flags & OPT_PDFOBJ_NO_PREDICTOR) {
+    settings.object.enable_predictor = 0;
+  } else {
+    settings.object.enable_predictor = 1;
+  }
   /* Set default paper size here so that all page's can inherite it.
    * annot_grow:    Margin of annotation.
    * bookmark_open: Miximal depth of open bookmarks.
    */
-  pdf_open_document(pdf_filename, do_encryption,
-                    paper_width, paper_height, annot_grow, bookmark_open,
-                    !(opt_flags & OPT_PDFDOC_NO_DEST_REMOVE));
+  if (landscape_mode) {
+    SWAP(paper_width, paper_height);
+  }  
+  settings.media_width        = paper_width;
+  settings.media_height       = paper_height;
+  settings.annot_grow_amount  = annot_grow;
+  settings.outline_open_depth = bookmark_open;
+  settings.check_gotos        = !(opt_flags & OPT_PDFDOC_NO_DEST_REMOVE);
 
-  /* Ignore_colors placed here since
-   * they are considered as device's capacity.
+  settings.device.dvi2pts     = dvi2pts;
+  settings.device.precision   = pdfdecimaldigits;
+  settings.device.ignore_colors = ignore_colors;
+
+  set_distiller_template(filter_template);
+  /* This must come before pdf_open_document where initialization
+   * of pdf_enc takes place.
    */
-  pdf_init_device(dvi2pts, pdfdecimaldigits, ignore_colors);
+  {
+    int version = pdf_version_major * 10 + pdf_version_minor;
+    pdf_set_version(version);
+  }
+  pdf_set_compression(compression_level);
+  if (enable_thumbnail)
+    pdf_doc_enable_manual_thumbnails();
+
+  /* Initialize PDF document creation routine. */
+  pdf_open_document(pdf_filename, ids, settings);
 
   if (opt_flags & OPT_CIDFONT_FIXEDPITCH)
     CIDFont_set_flags(CIDFONT_FORCE_FIXEDPITCH);
-
   /* Please move this to spc_init_specials(). */
   if (opt_flags & OPT_TPIC_TRANSPARENT_FILL)
     tpic_set_fill_mode(1);
+  if (translate_origin)
+    mps_set_translate_origin(1);
 
-  if (mp_mode) {
+  if (dpx_conf.compat_mode == dpx_mode_mpost_mode) {
     do_mps_pages();
   } else {
     do_dvi_pages();
   }
 
-  pdf_files_close();
-
-  /* Order of close... */
-  pdf_close_device  ();
-  /* pdf_close_document flushes XObject (image) and other resources. */
   pdf_close_document();
 
   pdf_close_fontmaps(); /* pdf_font may depend on fontmap. */
 
-  if (!mp_mode)
+  if (dpx_conf.compat_mode != dpx_mode_mpost_mode)
     dvi_close();
 
   MESG("\n");
