@@ -1,5 +1,5 @@
 /*   $Id$
- *   Copyright 1986-2016 Tomas Rokicki.
+ *   Copyright 1986-2018 Tomas Rokicki.
  *   This is dvips, a freely redistributable PostScript driver
  *   for dvi files. You may freely use, modify and/or distribute this
  *   program or any portion thereof.
@@ -15,7 +15,7 @@
 #define CD_IDX(i)  ((i>=MAX_2BYTES_CODE ? MAX_2BYTES_CODE : i))
 
 #define BANNER \
-"This is dvips(k) 5.997 Copyright 2017 Radical Eye Software"
+"This is dvips(k) 5.998 Copyright 2018 Radical Eye Software"
 #define BANNER2 "(www.radicaleye.com)"
 #ifdef KPATHSEA
 #include "config.h"
@@ -91,8 +91,10 @@ extern char *sprintf();
  */
 #if (defined(MSDOS) && !defined(DJGPP)) || (defined(OS2) && defined(_MSC_VER)) || defined(ATARIST)
 typedef long integer;
+typedef unsigned long uinteger;
 #else
 typedef int integer;
+typedef unsigned int uinteger;
 #endif
 #ifndef KPATHSEA
 typedef char boolean;
@@ -249,7 +251,7 @@ struct header_list {
    const char *Hname;
    char *precode;
    char *postcode;
-   char name[1];
+   char *name;
 };
 /*
  *   Some machines define putlong in their library.
