@@ -2,7 +2,7 @@
 ** BgColorSpecialHandler.cpp                                            **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,7 +18,6 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
 #include <algorithm>
 #include "BgColorSpecialHandler.hpp"
 #include "ColorSpecialHandler.hpp"
@@ -30,7 +29,7 @@ using namespace std;
 /** Collect all background color changes while preprocessing the DVI file.
  *  We need them in order to apply the correct background colors even if
  *  not all but only selected DVI pages are converted. */
-void BgColorSpecialHandler::preprocess (const char*, std::istream &is, SpecialActions &actions) {
+void BgColorSpecialHandler::preprocess (const string&, std::istream &is, SpecialActions &actions) {
 	Color color = ColorSpecialHandler::readColor(is);
 	unsigned pageno = actions.getCurrentPageNumber();
 	if (_pageColors.empty() || _pageColors.back().second != color) {
@@ -42,17 +41,11 @@ void BgColorSpecialHandler::preprocess (const char*, std::istream &is, SpecialAc
 }
 
 
-bool BgColorSpecialHandler::process (const char*, istream &, SpecialActions&) {
+bool BgColorSpecialHandler::process (const string&, istream&, SpecialActions&) {
 	return true;
 }
 
 
-static bool operator < (const pair<unsigned,Color> &pc1, const pair<unsigned,Color> &pc2) {
-	// order PageColor objects by page number
-	return pc1.first < pc2.first;
-}
-
-
 void BgColorSpecialHandler::dviBeginPage (unsigned pageno, SpecialActions &actions) {
 	// Ensure that the background color of the preceeding page is set as the
 	// default background color of the current page because this special affects
@@ -69,7 +62,7 @@ void BgColorSpecialHandler::dviBeginPage (unsigned pageno, SpecialActions &actio
 }
 
 
-const char** BgColorSpecialHandler::prefixes () const {
-	static const char *pfx[] = {"background", 0};
+vector<const char*> BgColorSpecialHandler::prefixes() const {
+	vector<const char*> pfx {"background"};
 	return pfx;
 }
