@@ -2,7 +2,7 @@
 ** DVIReader.cpp                                                        **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,7 +18,6 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
 #include <algorithm>
 #include <cstdarg>
 #include <fstream>
@@ -28,6 +27,7 @@
 #include "DVIReader.hpp"
 #include "Font.hpp"
 #include "FontManager.hpp"
+#include "HashFunction.hpp"
 #include "VectorStream.hpp"
 
 using namespace std;
@@ -69,7 +69,7 @@ void DVIReader::executeAll () {
 
 /** Reads and executes the commands of a single page.
  *  This methods stops reading after the page's eop command has been executed.
- *  @param[in] n number of page to be executed
+ *  @param[in] n number of page to be executed (1-based)
  *  @returns true if page was read successfully */
 bool DVIReader::executePage (unsigned n) {
 	clearStream();    // reset all status bits
@@ -133,14 +133,46 @@ void DVIReader::collectBopOffsets () {
 	_bopOffsets.push_back(tell());      // also add offset of postamble
 	readByte();                         // skip post command
 	uint32_t offset = readUnsigned(4);  // offset of final bop
-	while ((int32_t)offset > 0) {       // not yet on first bop?
+	while (int32_t(offset) != -1) {     // not yet on first bop?
 		_bopOffsets.push_back(offset);   // record offset
-		seek(offset+41);                 // skip bop command and the 10 \count values => now on offset of previous bop
-		offset = readUnsigned(4);
+		seek(offset);                    // now on previous bop
+		if (readByte() != OP_BOP)
+			throw DVIException("bop offset at "+to_string(offset)+" doesn't point to bop command" );
+		seek(40, ios::cur);              // skip the 10 \count values => now on offset of previous bop
+		uint32_t prevOffset = readUnsigned(4);
+		if ((prevOffset >= offset && int32_t(prevOffset) != -1))
+			throw DVIException("invalid bop offset at "+to_string(tell()-static_cast<streamoff>(4)));
+		offset = prevOffset;
 	}
 	reverse(_bopOffsets.begin(), _bopOffsets.end());
 }
 
+
+/** Computes a hash value for a given page. The hash algorithm is selected by
+ *  a HashFunction object which will also contain the resulting hash value if
+ *  this function returns true.
+ *  @param[in] pageno number of page to process (1-based)
+ *  @param[in,out] hashFunc hash function to use
+ *  @return true on success, hashFunc contains the resulting hash value */
+bool DVIReader::computePageHash (size_t pageno, HashFunction &hashFunc) {
+	if (pageno == 0 || pageno > numberOfPages())
+		return false;
+
+	hashFunc.reset();
+	clearStream();
+	seek(_bopOffsets[pageno-1]+45);  // now on first command after bop of selected page
+	const size_t BUFSIZE = 4096;
+	char buf[BUFSIZE];
+	size_t numBytes = numberOfPageBytes(pageno-1)-46;  // number of bytes excluding bop and eop
+	while (numBytes > 0) {
+		getInputStream().read(buf, min(numBytes, BUFSIZE));
+		hashFunc.update(buf, getInputStream().gcount());
+		numBytes -= getInputStream().gcount();
+	}
+	return true;
+}
+
+
 /////////////////////////////////////
 
 /** Reads and executes DVI preamble command.
@@ -175,7 +207,7 @@ void DVIReader::cmdPost (int) {
 	uint32_t pageWidth  = readUnsigned(4); // width of widest page in dvi units
 	uint16_t stackDepth = readUnsigned(2); // max. stack depth required
 	uint16_t numPages = readUnsigned(2);
-	if (numPages != numberOfPages())
+	if (numPages != (numberOfPages() & 0xffff))
 		throw DVIException("page count in postamble doesn't match actual number of pages");
 
 	// 1 dviunit * num/den == multiples of 0.0000001m
@@ -284,7 +316,7 @@ void DVIReader::cmdSetChar0 (int c) {
 	Font *font = FontManager::instance().getFont(_currFontNum);
 	dviSetChar0(c, font); // template method that may trigger further actions
 	putVFChar(font, c);   // further character processing if current font is a virtual font
-	moveRight(font->charWidth(c)*font->scaleFactor()*_mag/1000.0);
+	moveRight(font->charWidth(c)*font->scaleFactor()*_mag/1000.0, MoveMode::SETCHAR);
 }
 
 
@@ -301,7 +333,7 @@ void DVIReader::cmdSetChar (int len) {
 	Font *font = FontManager::instance().getFont(_currFontNum);
 	dviSetChar(c, font); // template method that may trigger further actions
 	putVFChar(font, c);  // further character processing if current font is a virtual font
-	moveRight(font->charWidth(c)*font->scaleFactor()*_mag/1000.0);
+	moveRight(font->charWidth(c)*font->scaleFactor()*_mag/1000.0, MoveMode::SETCHAR);
 }
 
 
@@ -331,7 +363,7 @@ void DVIReader::cmdSetRule (int) {
 	double height = _dvi2bp*readSigned(4);
 	double width  = _dvi2bp*readSigned(4);
 	dviSetRule(height, width);
-	moveRight(width);
+	moveRight(width, MoveMode::CHANGEPOS);
 }
 
 
@@ -348,7 +380,7 @@ void DVIReader::cmdPutRule (int) {
 }
 
 
-void DVIReader::moveRight (double dx) {
+void DVIReader::moveRight (double dx, MoveMode) {
 	switch (_dviState.d) {
 		case WritingMode::LR: _dviState.h += dx; break;
 		case WritingMode::TB: _dviState.v += dx; break;
@@ -357,7 +389,7 @@ void DVIReader::moveRight (double dx) {
 }
 
 
-void DVIReader::moveDown (double dy) {
+void DVIReader::moveDown (double dy, MoveMode) {
 	switch (_dviState.d) {
 		case WritingMode::LR: _dviState.v += dy; break;
 		case WritingMode::TB: _dviState.h -= dy; break;
@@ -368,29 +400,29 @@ void DVIReader::moveDown (double dy) {
 
 void DVIReader::cmdRight (int len) {
 	double dx = _dvi2bp*readSigned(len);
-	moveRight(dx);
+	moveRight(dx, MoveMode::CHANGEPOS);
 	dviRight(dx);
 }
 
 
 void DVIReader::cmdDown (int len) {
 	double dy = _dvi2bp*readSigned(len);
-	moveDown(dy);
+	moveDown(dy, MoveMode::CHANGEPOS);
 	dviDown(dy);
 }
 
 
 void DVIReader::cmdNop (int) {dviNop();}
-void DVIReader::cmdX0 (int)  {moveRight(_dviState.x); dviX0();}
-void DVIReader::cmdY0 (int)  {moveDown(_dviState.y); dviY0();}
-void DVIReader::cmdW0 (int)  {moveRight(_dviState.w); dviW0();}
-void DVIReader::cmdZ0 (int)  {moveDown(_dviState.z); dviZ0();}
+void DVIReader::cmdX0 (int)  {moveRight(_dviState.x, MoveMode::CHANGEPOS); dviX0();}
+void DVIReader::cmdY0 (int)  {moveDown(_dviState.y, MoveMode::CHANGEPOS); dviY0();}
+void DVIReader::cmdW0 (int)  {moveRight(_dviState.w, MoveMode::CHANGEPOS); dviW0();}
+void DVIReader::cmdZ0 (int)  {moveDown(_dviState.z, MoveMode::CHANGEPOS); dviZ0();}
 
 
 void DVIReader::cmdX (int len) {
 	double dx = _dvi2bp*readSigned(len);
 	_dviState.x = dx;
-	moveRight(dx);
+	moveRight(dx, MoveMode::CHANGEPOS);
 	dviX(dx);
 }
 
@@ -398,7 +430,7 @@ void DVIReader::cmdX (int len) {
 void DVIReader::cmdY (int len) {
 	double dy = _dvi2bp*readSigned(len);
 	_dviState.y = dy;
-	moveDown(dy);
+	moveDown(dy, MoveMode::CHANGEPOS);
 	dviY(dy);
 }
 
@@ -406,7 +438,7 @@ void DVIReader::cmdY (int len) {
 void DVIReader::cmdW (int len) {
 	double dx = _dvi2bp*readSigned(len);
 	_dviState.w = dx;
-	moveRight(dx);
+	moveRight(dx, MoveMode::CHANGEPOS);
 	dviW(dx);
 }
 
@@ -414,7 +446,7 @@ void DVIReader::cmdW (int len) {
 void DVIReader::cmdZ (int len) {
 	double dy = _dvi2bp*readSigned(len);
 	_dviState.z = dy;
-	moveDown(dy);
+	moveDown(dy, MoveMode::CHANGEPOS);
 	dviZ(dy);
 }
 
@@ -611,7 +643,7 @@ void DVIReader::cmdXGlyphArray (int) {
 		dviXGlyphArray(dx, dy, glyphs, *font);
 	else
 		throw DVIException("missing setfont prior to xglypharray");
-	moveRight(width);
+	moveRight(width, MoveMode::SETCHAR);
 }
 
 
@@ -626,7 +658,7 @@ void DVIReader::cmdXGlyphString (int) {
 		dviXGlyphString(dx, glyphs, *font);
 	else
 		throw DVIException("missing setfont prior to xglyphstring");
-	moveRight(width);
+	moveRight(width, MoveMode::SETCHAR);
 }
 
 
@@ -648,7 +680,7 @@ void DVIReader::cmdXTextAndGlyphs (int) {
 		dviXTextAndGlyphs(x, y, chars, glyphs, *font);
 	else
 		throw DVIException("missing setfont prior to xtextandglyphs");
-	moveRight(width);
+	moveRight(width, MoveMode::SETCHAR);
 }
 
 
