@@ -2,7 +2,7 @@
 ** DVIToSVG.cpp                                                         **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -22,11 +22,13 @@
 #include <cstdlib>
 #include <ctime>
 #include <fstream>
+#include <iomanip>
 #include <set>
 #include <sstream>
 #include "Calculator.hpp"
 #include "DVIToSVG.hpp"
 #include "DVIToSVGActions.hpp"
+#include "FileSystem.hpp"
 #include "Font.hpp"
 #include "FontManager.hpp"
 #include "GlyphTracerMessages.hpp"
@@ -37,7 +39,9 @@
 #include "PreScanDVIReader.hpp"
 #include "SignalHandler.hpp"
 #include "SVGOutput.hpp"
+#include "utility.hpp"
 #include "version.hpp"
+#include "XXHashFunction.hpp"
 
 ///////////////////////////////////
 // special handlers
@@ -49,13 +53,13 @@
 #include "HtmlSpecialHandler.hpp"
 #include "PapersizeSpecialHandler.hpp"
 #include "PdfSpecialHandler.hpp"
+#include "TpicSpecialHandler.hpp"
 #ifndef HAVE_LIBGS
 	#include "NoPsSpecialHandler.hpp"
 #endif
 #ifndef DISABLE_GS
 	#include "PsSpecialHandler.hpp"
 #endif
-#include "TpicSpecialHandler.hpp"
 
 ///////////////////////////////////
 
@@ -66,6 +70,7 @@ using namespace std;
  *   0 : only trace actually required glyphs */
 char DVIToSVG::TRACE_MODE = 0;
 bool DVIToSVG::COMPUTE_PROGRESS = false;
+DVIToSVG::HashSettings DVIToSVG::PAGE_HASH_SETTINGS;
 
 
 DVIToSVG::DVIToSVG (istream &is, SVGOutputBase &out) : DVIReader(is), _out(out)
@@ -75,20 +80,15 @@ DVIToSVG::DVIToSVG (istream &is, SVGOutputBase &out) : DVIReader(is), _out(out)
 	_pageByte = 0;
 	_prevXPos = _prevYPos = numeric_limits<double>::min();
 	_prevWritingMode = WritingMode::LR;
-	_actions = new DVIToSVGActions(*this, _svg);
-}
-
-
-DVIToSVG::~DVIToSVG () {
-	delete _actions;
+	_actions = util::make_unique<DVIToSVGActions>(*this, _svg);
 }
 
 
 /** Starts the conversion process.
  *  @param[in] first number of first page to convert
  *  @param[in] last number of last page to convert
- *  @param[out] pageinfo (number of converted pages, number of total pages) */
-void DVIToSVG::convert (unsigned first, unsigned last, pair<int,int> *pageinfo) {
+ *  @param[in] hashFunc pointer to function to be used to compute page hashes */
+void DVIToSVG::convert (unsigned first, unsigned last, HashFunction *hashFunc) {
 	if (first > last)
 		swap(first, last);
 	if (first > numberOfPages()) {
@@ -99,28 +99,59 @@ void DVIToSVG::convert (unsigned first, unsigned last, pair<int,int> *pageinfo)
 		throw DVIException(oss.str());
 	}
 	last = min(last, numberOfPages());
+	bool computeHashes = (hashFunc && !_out.ignoresHashes());
+	string shortenedOptHash = XXH32HashFunction(PAGE_HASH_SETTINGS.optionsHash()).digestString();
 	for (unsigned i=first; i <= last; ++i) {
-		executePage(i);
-		_svg.removeRedundantElements();
-		embedFonts(_svg.rootNode());
-		bool success = _svg.write(_out.getPageStream(currentPageNumber(), numberOfPages()));
-		string fname = _out.filename(i, numberOfPages());
-		if (fname.empty())
-			fname = "<stdout>";
-		if (success)
-			Message::mstream(false, Message::MC_PAGE_WRITTEN) << "\noutput written to " << fname << '\n';
-		else
-			Message::wstream(true) << "failed to write output to " << fname << '\n';
-		_svg.reset();
-		static_cast<DVIToSVGActions*>(_actions)->reset();
-	}
-	if (pageinfo) {
-		pageinfo->first = last-first+1;
-		pageinfo->second = numberOfPages();
+		string dviHash, combinedHash;
+		if (computeHashes) {
+			computePageHash(i, *hashFunc);
+			dviHash = hashFunc->digestString();
+			hashFunc->update(PAGE_HASH_SETTINGS.optionsHash());
+			combinedHash = hashFunc->digestString();
+		}
+		const SVGOutput::HashTriple hashTriple(dviHash, shortenedOptHash, combinedHash);
+		string fname = _out.filename(i, numberOfPages(), hashTriple);
+		if (!dviHash.empty() && !PAGE_HASH_SETTINGS.isSet(HashSettings::P_REPLACE) && FileSystem::exists(fname)) {
+			Message::mstream(false, Message::MC_PAGE_NUMBER) << "skipping page " << i;
+			Message::mstream().indent(1);
+			Message::mstream(false, Message::MC_PAGE_WRITTEN) << "\nfile " << fname << " exists\n";
+			Message::mstream().indent(0);
+		}
+		else {
+			executePage(i);
+			_svg.removeRedundantElements();
+			embedFonts(_svg.rootNode());
+			bool success = _svg.write(_out.getPageStream(currentPageNumber(), numberOfPages(), hashTriple));
+			if (fname.empty())
+				fname = "<stdout>";
+			if (success)
+				Message::mstream(false, Message::MC_PAGE_WRITTEN) << "\noutput written to " << fname << '\n';
+			else
+				Message::wstream(true) << "failed to write output to " << fname << '\n';
+			_svg.reset();
+			_actions->reset();
+		}
 	}
 }
 
 
+/** Creates a HashFunction object for a given algorithm name.
+ *  @param[in] algo name of hash algorithm
+ *  @return pointer to hash function
+ *  @throw MessageException if algorithm name is invalid or not supported */
+static unique_ptr<HashFunction> create_hash_function (const string &algo) {
+	if (auto hashFunc = HashFunction::create(algo))
+		return hashFunc;
+
+	string msg = "unknown hash algorithm '"+algo+"' (supported algorithms: ";
+	for (const string &name : HashFunction::supportedAlgorithms())
+		msg += name + ", ";
+	msg.pop_back();
+	msg.back() = ')';
+	throw MessageException(msg);
+}
+
+
 /** Convert DVI pages specified by a page range string.
  *  @param[in] rangestr string describing the pages to convert
  *  @param[out] pageinfo (number of converted pages, number of total pages) */
@@ -130,7 +161,7 @@ void DVIToSVG::convert (const string &rangestr, pair<int,int> *pageinfo) {
 		throw MessageException("invalid page range format");
 
 	Message::mstream(false, Message::MC_PAGE_NUMBER) << "pre-processing DVI file (format version "  << getDVIVersion() << ")\n";
-	if (DVIToSVGActions *actions = dynamic_cast<DVIToSVGActions*>(_actions)) {
+	if (DVIToSVGActions *actions = dynamic_cast<DVIToSVGActions*>(_actions.get())) {
 		PreScanDVIReader prescan(getInputStream(), actions);
 		actions->setDVIReader(prescan);
 		prescan.executeAllPages();
@@ -138,8 +169,12 @@ void DVIToSVG::convert (const string &rangestr, pair<int,int> *pageinfo) {
 		SpecialManager::instance().notifyPreprocessingFinished();
 	}
 
+	unique_ptr<HashFunction> hashFunc;
+	if (!PAGE_HASH_SETTINGS.algorithm().empty())  // name of hash algorithm present?
+		hashFunc = create_hash_function(PAGE_HASH_SETTINGS.algorithm());
+
 	for (const auto &range : ranges)
-		convert(range.first, range.second);
+		convert(range.first, range.second, hashFunc.get());
 	if (pageinfo) {
 		pageinfo->first = ranges.numberOfPages();
 		pageinfo->second = numberOfPages();
@@ -147,6 +182,37 @@ void DVIToSVG::convert (const string &rangestr, pair<int,int> *pageinfo) {
 }
 
 
+/** Writes the hash values of a selected set of pages to an output stream.
+ *  @param[in] rangestr string describing the pages to convert
+ *  @param[in,out] os stream the output is written to */
+void DVIToSVG::listHashes (const string &rangestr, std::ostream &os) {
+	PageRanges ranges;
+	if (!ranges.parse(rangestr, numberOfPages()))
+		throw MessageException("invalid page range format");
+
+	XXH32HashFunction xxh32;
+	auto hashFunc = create_hash_function(PAGE_HASH_SETTINGS.algorithm());
+	int width1 = util::ilog10(numberOfPages())+1;
+	int width2 = hashFunc->digestSize()*2;
+	int spaces1 = width1+2+(width2-3)/2;
+	int spaces2 = width1+2+width2+2-spaces1-3+(width2-7)/2;
+	string shortenedOptHash = XXH32HashFunction(PAGE_HASH_SETTINGS.optionsHash()).digestString();
+	os << string(spaces1, ' ') << "DVI"
+		<< string(spaces2, ' ') << "DVI+opt\n";
+	for (const auto &range : ranges) {
+		for (int i=range.first; i <= range.second; i++) {
+			computePageHash(i, *hashFunc);
+			os << setw(width1) << i;
+			os << ": " << hashFunc->digestString();
+			hashFunc->update(PAGE_HASH_SETTINGS.optionsHash());
+			os << ", " << hashFunc->digestString() << '\n';
+		}
+	}
+	os << "hash algorithm: " << PAGE_HASH_SETTINGS.algorithm()
+		<< ", options hash: " << shortenedOptHash << '\n';
+}
+
+
 int DVIToSVG::executeCommand () {
 	SignalHandler::instance().check();
 	const streampos cmdpos = tell();
@@ -171,13 +237,13 @@ int DVIToSVG::executeCommand () {
  *  @param[in] pageno physical page number (1 = first page)
  *  @param[in] c contains information about the page (page number etc.) */
 void DVIToSVG::enterBeginPage (unsigned pageno, const vector<int32_t> &c) {
-	if (dynamic_cast<DVIToSVGActions*>(_actions)) {
+	if (dynamic_cast<DVIToSVGActions*>(_actions.get())) {
 		Message::mstream().indent(0);
 		Message::mstream(false, Message::MC_PAGE_NUMBER) << "processing page " << pageno;
 		if (pageno != (unsigned)c[0])  // Does page number shown on page differ from physical page number?
 			Message::mstream(false) << " [" << c[0] << ']';
 		Message::mstream().indent(1);
-		_svg.appendToDoc(new XMLCommentNode(" This file was generated by dvisvgm " + string(PROGRAM_VERSION) + " "));
+		_svg.appendToDoc(util::make_unique<XMLCommentNode>(" This file was generated by dvisvgm " + string(PROGRAM_VERSION) + " "));
 	}
 }
 
@@ -185,15 +251,14 @@ void DVIToSVG::enterBeginPage (unsigned pageno, const vector<int32_t> &c) {
 /** This template method is called by DVIReader::cmdEop() after
  *  executing the EOP actions. */
 void DVIToSVG::leaveEndPage (unsigned) {
-	if (!dynamic_cast<DVIToSVGActions*>(_actions))
+	if (!dynamic_cast<DVIToSVGActions*>(_actions.get()))
 		return;
 
 	// set bounding box and apply page transformations
 	BoundingBox bbox = _actions->bbox();  // bounding box derived from the DVI commands executed
 	if (_bboxFormatString == "min" || _bboxFormatString == "preview" || _bboxFormatString == "papersize") {
-		Matrix matrix;
-		getPageTransformation(matrix);
-		bbox.transform(matrix);
+		bbox.unlock();
+		bbox.transform(getPageTransformation());
 	}
 	else if (_bboxFormatString == "dvi") {
 		// center page content
@@ -212,18 +277,15 @@ void DVIToSVG::leaveEndPage (unsigned) {
 				// convention: DVI position (0,0) equals (1in, 1in) relative
 				// to the upper left vertex of the page (see DVI specification)
 				const double border = -72;
-				bbox = BoundingBox(border, border, size.widthInBP()+border, size.heightInBP()+border);
+				bbox = BoundingBox(border, border, size.width().bp()+border, size.height().bp()+border);
 			}
 		}
 		else { // set/modify bounding box by explicitly given values
 			try {
-				vector<Length> lengths;
-				BoundingBox::extractLengths(_bboxFormatString, lengths);
+				vector<Length> lengths = BoundingBox::extractLengths(_bboxFormatString);
 				if (lengths.size() == 1 || lengths.size() == 2) {  // relative box size?
 					// apply the page transformation and adjust the bbox afterwards
-					Matrix matrix;
-					getPageTransformation(matrix);
-					bbox.transform(matrix);
+					bbox.transform(getPageTransformation());
 				}
 				bbox.set(lengths);
 			}
@@ -247,29 +309,28 @@ void DVIToSVG::leaveEndPage (unsigned) {
 }
 
 
-void DVIToSVG::getPageTransformation(Matrix &matrix) const {
-	if (_transCmds.empty())
-		matrix.set(1);  // unity matrix
-	else {
+Matrix DVIToSVG::getPageTransformation () const {
+	Matrix matrix(1); // unity matrix
+	if (!_transCmds.empty()) {
 		Calculator calc;
 		if (_actions) {
-			const double bp2pt = 72.27/72;
+			const double bp2pt = (1_bp).pt();
 			BoundingBox &bbox = _actions->bbox();
 			calc.setVariable("ux", bbox.minX()*bp2pt);
 			calc.setVariable("uy", bbox.minY()*bp2pt);
 			calc.setVariable("w",  bbox.width()*bp2pt);
 			calc.setVariable("h",  bbox.height()*bp2pt);
 		}
-		calc.setVariable("pt", 1);
-		calc.setVariable("in", 72.27);
-		calc.setVariable("cm", 72.27/2.54);
-		calc.setVariable("mm", 72.27/25.4);
+		// add constants for length units to calculator
+		for (auto unit : Length::getUnits())
+			calc.setVariable(unit.first, Length(1, unit.second).pt());
 		matrix.set(_transCmds, calc);
 	}
+	return matrix;
 }
 
 
-static void collect_chars (map<const Font*, set<int>> &fontmap) {
+static void collect_chars (unordered_map<const Font*, set<int>> &fontmap) {
 	for (const auto &entry : fontmap) {
 		if (entry.first->uniqueFont() != entry.first) {
 			for (int c : entry.second)
@@ -282,18 +343,16 @@ static void collect_chars (map<const Font*, set<int>> &fontmap) {
 /** Adds the font information to the SVG tree.
  *  @param[in] svgElement the font nodes are added to this node */
 void DVIToSVG::embedFonts (XMLElementNode *svgElement) {
-	if (!svgElement)
-		return;
-	if (!_actions)  // no dvi actions => no chars written => no fonts to embed
+	if (!svgElement || !_actions) // no dvi actions => no chars written => no fonts to embed
 		return;
 
-	const DVIToSVGActions *svgActions = static_cast<DVIToSVGActions*>(_actions);
-	map<const Font*,set<int>> &usedCharsMap = svgActions->getUsedChars();
+	const DVIToSVGActions *svgActions = static_cast<DVIToSVGActions*>(_actions.get());
+	auto &usedCharsMap = svgActions->getUsedChars();
 
 	collect_chars(usedCharsMap);
 
 	GlyphTracerMessages messages;
-	set<const Font*> tracedFonts;  // collect unique fonts already traced
+	unordered_set<const Font*> tracedFonts;  // collect unique fonts already traced
 	for (const auto &fontchar : usedCharsMap) {
 		const Font *font = fontchar.first;
 		if (const PhysicalFont *ph_font = dynamic_cast<const PhysicalFont*>(font)) {
@@ -324,32 +383,28 @@ void DVIToSVG::embedFonts (XMLElementNode *svgElement) {
  *  @param[in] pswarning if true, shows warning about disabled PS support
  *  @return the SpecialManager that handles special statements */
 void DVIToSVG::setProcessSpecials (const char *ignorelist, bool pswarning) {
-	if (ignorelist && strcmp(ignorelist, "*") == 0) { // ignore all specials?
+	if (ignorelist && strcmp(ignorelist, "*") == 0)  // ignore all specials?
 		SpecialManager::instance().unregisterHandlers();
-	}
 	else {
 		// add special handlers
-		SpecialHandler *handlers[] = {
-			0,                           // placeholder for PsSpecialHandler
-			new BgColorSpecialHandler,   // handles background color special
-			new ColorSpecialHandler,     // handles color specials
-			new DvisvgmSpecialHandler,   // handles raw SVG embeddings
-			new EmSpecialHandler,        // handles emTeX specials
-			new HtmlSpecialHandler,      // handles hyperref specials
-			new PapersizeSpecialHandler, // handles papersize special
-			new PdfSpecialHandler,       // handles pdf specials
-			new TpicSpecialHandler,      // handles tpic specials
-			0
-		};
-		SpecialHandler **p = handlers;
+		vector<unique_ptr<SpecialHandler>> handlers;
+		handlers.emplace_back(util::make_unique<BgColorSpecialHandler>());   // handles background color special
+		handlers.emplace_back(util::make_unique<ColorSpecialHandler>());     // handles color specials
+		handlers.emplace_back(util::make_unique<DvisvgmSpecialHandler>());   // handles raw SVG embeddings
+		handlers.emplace_back(util::make_unique<EmSpecialHandler>());        // handles emTeX specials
+		handlers.emplace_back(util::make_unique<HtmlSpecialHandler>());      // handles hyperref specials
+		handlers.emplace_back(util::make_unique<PapersizeSpecialHandler>()); // handles papersize special
+		handlers.emplace_back(util::make_unique<PdfSpecialHandler>());       // handles pdf specials
+		handlers.emplace_back(util::make_unique<TpicSpecialHandler>());      // handles tpic specials
 #ifndef DISABLE_GS
 		if (Ghostscript().available())
-			*p = new PsSpecialHandler;
+			handlers.emplace_back(util::make_unique<PsSpecialHandler>());     // handles PostScript specials
 		else
 #endif
 		{
 #ifndef HAVE_LIBGS
-			*p = new NoPsSpecialHandler; // dummy PS special handler that only prints warning messages
+			// dummy PS special handler that only prints warning messages
+			handlers.emplace_back(util::make_unique<NoPsSpecialHandler>());
 			if (pswarning) {
 #ifdef DISABLE_GS
 				Message::wstream() << "processing of PostScript specials has been disabled permanently\n";
@@ -360,7 +415,7 @@ void DVIToSVG::setProcessSpecials (const char *ignorelist, bool pswarning) {
 #endif
 		}
 		SpecialManager::instance().unregisterHandlers();
-		SpecialManager::instance().registerHandlers(p, ignorelist);
+		SpecialManager::instance().registerHandlers(handlers, ignorelist);
 	}
 }
 
@@ -370,24 +425,24 @@ string DVIToSVG::getSVGFilename (unsigned pageno) const {
 }
 
 
-void DVIToSVG::moveRight (double dx) {
-	DVIReader::moveRight(dx);
+void DVIToSVG::moveRight (double dx, MoveMode mode) {
+	DVIReader::moveRight(dx, mode);
 	if (_actions) {
 		if (dviState().d == WritingMode::LR)
-			_actions->moveToX(dviState().h+_tx);
+			_actions->moveToX(dviState().h+_tx, mode == MoveMode::CHANGEPOS);
 		else
-			_actions->moveToY(dviState().v+_ty);
+			_actions->moveToY(dviState().v+_ty, mode == MoveMode::CHANGEPOS);
 	}
 }
 
 
-void DVIToSVG::moveDown (double dy) {
-	DVIReader::moveDown(dy);
+void DVIToSVG::moveDown (double dy, MoveMode mode) {
+	DVIReader::moveDown(dy, mode);
 	if (_actions) {
 		if (dviState().d == WritingMode::LR)
-			_actions->moveToY(dviState().v+_ty);
+			_actions->moveToY(dviState().v+_ty, mode == MoveMode::CHANGEPOS);
 		else
-			_actions->moveToX(dviState().h+_tx);
+			_actions->moveToX(dviState().h+_tx, mode == MoveMode::CHANGEPOS);
 	}
 }
 
@@ -448,9 +503,9 @@ void DVIToSVG::dviPutRule (double height, double width) {
 void DVIToSVG::dviPop () {
 	if (_actions) {
 		if (_prevXPos != dviState().h+_tx)
-			_actions->moveToX(dviState().h + _tx);
+			_actions->moveToX(dviState().h + _tx, true);  // force setting the SVG position
 		if (_prevYPos != dviState().v+_ty)
-			_actions->moveToY(dviState().v + _ty);
+			_actions->moveToY(dviState().v + _ty, true);  // force setting the SVG position
 		if (_prevWritingMode != dviState().d)
 			_actions->setTextOrientation(dviState().d != WritingMode::LR);
 	}
@@ -494,3 +549,26 @@ void DVIToSVG::dviXGlyphString (vector<double> &dx, vector<uint16_t> &glyphs, co
 void DVIToSVG::dviXTextAndGlyphs (vector<double> &dx, vector<double> &dy, vector<uint16_t>&, vector<uint16_t> &glyphs, const Font &font) {
 	dviXGlyphArray(dx, dy, glyphs, font);
 }
+
+///////////////////////////////////////////////////////////////
+
+/** Parses a string consisting of comma-separated words, and assigns
+ *  the values to the hash settings. */
+void DVIToSVG::HashSettings::setParameters (const string &paramstr) {
+	auto paramnames = util::split(paramstr, ",");
+	map<string, Parameter> paramMap = {
+		{"list", P_LIST},
+		{"replace", P_REPLACE}
+	};
+	for (string &name : paramnames) {
+		name = util::trim(name);
+		auto it = paramMap.find(name);
+		if (it != paramMap.end())
+			_params.insert(it->second);
+		else if (_algo.empty() && HashFunction::isSupportedAlgorithm(name))
+			_algo = name;
+	}
+	// set default hash algorithm if none is given
+	if (_algo.empty())
+		_algo = "xxh64";
+}
