@@ -2,7 +2,7 @@
 ** EPSFile.cpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,7 +18,6 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
 #include <cstring>
 #include <istream>
 #include <limits>
@@ -57,8 +56,7 @@ static size_t getline (istream &is, char *line, size_t n) {
 }
 
 
-EPSFile::EPSFile (const std::string& fname) : _ifs(fname.c_str(), ios::binary), _headerValid(false), _offset(0), _pslength(0)
-{
+EPSFile::EPSFile (const string &fname) : _ifs(fname, ios::binary) {
 	if (_ifs) {
 		if (getUInt32(_ifs) != 0xC6D3D0C5)  // no binary header present?
 			_ifs.seekg(0);                   // go back to the first byte
@@ -85,10 +83,10 @@ istream& EPSFile::istream () const {
 }
 
 
-/** Extracts the bounding box information from the DSC header/footer (if present)
- *  @param[out] box the extracted bounding box
- *  @return true if %%BoundingBox data could be read successfully */
-bool EPSFile::bbox (BoundingBox &box) const {
+/** Extracts the bounding box information from the DSC header/footer (if present).
+ *  @return the extracted bounding box */
+BoundingBox EPSFile::bbox () const {
+	BoundingBox box;
 	std::istream &is = EPSFile::istream();
 	if (is) {
 		char buf[64];
@@ -106,10 +104,10 @@ bool EPSFile::bbox (BoundingBox &box) const {
 						ir.parseInt(val[i]);
 					}
 					box = BoundingBox(val[0], val[1], val[2], val[3]);
-					return true;
+					break;
 				}
 			}
 		}
 	}
-	return false;
+	return box;
 }
