@@ -2,7 +2,7 @@
 ** FontMap.cpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,7 +18,6 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
 #include <algorithm>
 #include <cstring>
 #include <fstream>
@@ -32,6 +31,7 @@
 #include "MapLine.hpp"
 #include "Message.hpp"
 #include "Subfont.hpp"
+#include "utility.hpp"
 
 using namespace std;
 
@@ -47,7 +47,7 @@ FontMap& FontMap::instance() {
  *  @param[in] mode selects how to integrate the map file entries into the global map tree
  *  @return true if file could be opened */
 bool FontMap::read (const string &fname, FontMap::Mode mode) {
-	ifstream ifs(fname.c_str());
+	ifstream ifs(fname);
 	if (!ifs)
 		return false;
 
@@ -164,12 +164,12 @@ bool FontMap::append (const MapLine &mapline) {
 			if (mapline.sfd())
 				mapline.sfd()->subfonts(subfonts);
 			else
-				subfonts.push_back(0);
+				subfonts.push_back(nullptr);
 			for (Subfont *subfont : subfonts) {
 				string fontname = mapline.texname()+(subfont ? subfont->id() : "");
 				auto it = _entries.find(fontname);
 				if (it == _entries.end()) {
-					_entries[fontname].reset(new Entry(mapline, subfont));
+					_entries.emplace(fontname, util::make_unique<Entry>(mapline, subfont));
 					appended = true;
 				}
 			}
@@ -193,12 +193,12 @@ bool FontMap::replace (const MapLine &mapline) {
 	if (mapline.sfd())
 		mapline.sfd()->subfonts(subfonts);
 	else
-		subfonts.push_back(0);
+		subfonts.push_back(nullptr);
 	for (Subfont *subfont : subfonts) {
 		string fontname = mapline.texname()+(subfont ? subfont->id() : "");
 		auto it = _entries.find(fontname);
 		if (it == _entries.end())
-			_entries[fontname].reset(new Entry(mapline, subfont));
+			_entries.emplace(fontname, util::make_unique<Entry>(mapline, subfont));
 		else if (!it->second->locked)
 			*it->second = Entry(mapline, subfont);
 	}
@@ -217,7 +217,7 @@ bool FontMap::remove (const MapLine &mapline) {
 		if (mapline.sfd())
 			mapline.sfd()->subfonts(subfonts);
 		else
-			subfonts.push_back(0);
+			subfonts.push_back(nullptr);
 		for (const Subfont *subfont : subfonts) {
 			string fontname = mapline.texname()+(subfont ? subfont->id() : "");
 			auto it = _entries.find(fontname);
@@ -245,7 +245,7 @@ void FontMap::readdir (const string &dirname) {
 	while (const char *fname = dir.read(Directory::ET_FILE)) {
 		if (strlen(fname) >= 4 && strcmp(fname+strlen(fname)-4, ".map") == 0) {
 			string path = dirname + "/" + fname;
-			read(path.c_str());
+			read(path);
 		}
 	}
 }
@@ -257,7 +257,7 @@ void FontMap::readdir (const string &dirname) {
 const FontMap::Entry* FontMap::lookup (const string &fontname) const {
 	auto it = _entries.find(fontname);
 	if (it == _entries.end())
-		return 0;
+		return nullptr;
 	return it->second.get();
 }
 
