@@ -25,6 +25,7 @@
 #include <istream>
 #include <string>
 #include <vector>
+#include "BoundingBox.hpp"
 #include "Ghostscript.hpp"
 #include "InputReader.hpp"
 #include "MessageException.hpp"
@@ -55,7 +56,6 @@ struct PSActions {
 	virtual void makepattern (std::vector<double> &p) =0;
 	virtual void moveto (std::vector<double> &p) =0;
 	virtual void newpath (std::vector<double> &p) =0;
-	virtual void pdfpagebox (std::vector<double> &p) =0;
 	virtual void querypos (std::vector<double> &p) =0;
 	virtual void restore (std::vector<double> &p) =0;
 	virtual void rotate (std::vector<double> &p) =0;
@@ -101,6 +101,8 @@ class PSInterpreter {
 		void limit (size_t max_bytes)          {_bytesToRead = max_bytes;}
 		void setFilter (PSFilter *filter)      {_filter = filter;}
 		PSActions* setActions (PSActions *actions);
+		int pdfPageCount (const std::string &fname);
+		BoundingBox pdfPageBox (const std::string &fname, int pageno);
 		const std::vector<std::string>& rawData () const {return _rawData;}
 
 	protected:
