@@ -2,7 +2,7 @@
 ** PathClipper.cpp                                                      **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,16 +18,15 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
 #include "Bezier.hpp"
 #include "PathClipper.hpp"
 
 using namespace std;
 using namespace ClipperLib;
 
-typedef ClipperLib::Path Polygon;
-typedef ClipperLib::Paths Polygons;
-typedef PathClipper::CurvedPath CurvedPath;
+using Polygon = ClipperLib::Path;
+using Polygons = ClipperLib::Paths;
+using CurvedPath = PathClipper::CurvedPath;
 
 const int SCALE_FACTOR = 1000;
 
@@ -192,12 +191,6 @@ static int32_t segment_id (const IntPoint &p1, const IntPoint &p2, double &t1, d
 }
 
 
-inline int32_t edge_id (const IntPoint &p1, const IntPoint &p2) {
-	double t=0;
-	return segment_id(p1, p2, t, t);
-}
-
-
 /** This function expects 3 colinear points p1, p2, and q, where q lies between p1 and p2,
  *  i.e. q divides the line \f$ \overline{p_1 p_2} \f$ somewhere. The function returns
  *  the corresponding division ratio. */
