@@ -2,7 +2,7 @@
 ** RangeMap.cpp                                                         **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -125,11 +125,11 @@ void RangeMap::addRange (uint32_t cmin, uint32_t cmax, uint32_t vmin) {
 void RangeMap::adaptNeighbors (Ranges::iterator it) {
 	if (it != _ranges.end()) {
 		// adapt left neighbor
-		if (it != _ranges.begin() && it->min() <= (it-1)->max()) {
-			Ranges::iterator lit = it-1;  // points to left neighbor
+		Ranges::iterator lit = it-1;    // points to left neighbor
+		if (it != _ranges.begin() && it->min() <= lit->max()) {
 			bool left_neighbor_valid = (it->min() > 0 && it->min()-1 >= lit->min());
-			if (left_neighbor_valid)      // is adapted left neighbor valid?
-				lit->max(it->min()-1);     // => assign new max value
+			if (left_neighbor_valid)     // is adapted left neighbor valid?
+				lit->max(it->min()-1);  // => assign new max value
 			if (!left_neighbor_valid || it->join(*lit))
 				it = _ranges.erase(lit);
 		}
@@ -176,6 +176,15 @@ uint32_t RangeMap::valueAt (uint32_t c) const {
 }
 
 
+/** Returns the number of values mapped. */
+size_t RangeMap::numValues () const {
+	size_t count=0;
+	for (const Range &range : _ranges)
+		count += range.max()-range.min()+1;
+	return count;
+}
+
+
 ostream& RangeMap::Range::write (ostream& os) const {
 	return os << '[' << _min << ',' << _max << "] => " << _minval;
 }
