@@ -2,7 +2,7 @@
 ** Terminal.cpp                                                         **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -30,7 +30,7 @@
 #endif
 
 #ifdef _WIN32
-#include <windows.h>
+#include "windows.hpp"
 #endif
 
 #include <cstdio>
