@@ -2,7 +2,7 @@
 ** ToUnicodeMap.cpp                                                     **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -36,17 +36,17 @@ bool ToUnicodeMap::addMissingMappings (uint32_t maxIndex) {
 	bool success=true;
 	// collect Unicode points already in assigned
 	NumericRanges<uint32_t> codepoints;
-	for (size_t i=0; i < size() && success; i++)
+	for (size_t i=0; i < numRanges() && success; i++)
 		codepoints.addRange(rangeAt(i).minval(), rangeAt(i).maxval());
 	// fill unmapped ranges
 	if (empty()) // no Unicode mapping present at all?
 		success = fillRange(1, maxIndex, 1, codepoints, true);
 	else {   // (partial) Unicode mapping present?
 		success = fillRange(1, rangeAt(0).min()-1, rangeAt(0).minval()-1, codepoints, false);
-		for (size_t i=0; i < size()-1 && success; i++)
+		for (size_t i=0; i < numRanges()-1 && success; i++)
 			success = fillRange(rangeAt(i).max()+1, rangeAt(i+1).min()-1, rangeAt(i).maxval()+1, codepoints, true);
 		if (success)
-			success = fillRange(rangeAt(size()-1).max()+1, maxIndex, rangeAt(size()-1).maxval()+1, codepoints, true);
+			success = fillRange(rangeAt(numRanges()-1).max()+1, maxIndex, rangeAt(numRanges()-1).maxval()+1, codepoints, true);
 	}
 	return success;
 }
