@@ -1,6 +1,7 @@
 /* elt-dirs.c: Translate a path element to its corresponding director{y,ies}.
 
-   Copyright 1993, 1994, 1995, 1996, 1997, 2008, 2009, 2010, 2011, 2016 Karl Berry.
+   Copyright 1993, 1994, 1995, 1996, 1997, 2008, 2009, 2010, 2011, 2016,
+   2017, 2018 Karl Berry.
    Copyright 1997, 1998, 1999, 2000, 2005 Olaf Weber.
 
    This library is free software; you can redistribute it and/or
@@ -100,7 +101,7 @@ static void expand_elt (kpathsea, str_llist_type *, string, unsigned);
 /* POST is a pointer into the original element (which may no longer be
    ELT) to just after the doubled DIR_SEP, perhaps to the null.  Append
    subdirectories of ELT (up to ELT_LENGTH, which must be a /) to
-   STR_LIST_PTR.  */
+   STR_LIST_PTR.  ELT must not be the empty string (or NULL).  */
 
 #ifdef WIN32
 /* Shared across recursive calls, it acts like a stack. */
@@ -127,8 +128,9 @@ do_subdir (kpathsea kpse, str_llist_type *str_list_ptr, string elt,
   /* Some old compilers don't allow aggregate initialization.  */
   name = fn_copy0 (elt, elt_length);
 
-  assert (IS_DIR_SEP_CH (elt[elt_length - 1])
-          || IS_DEVICE_SEP (elt[elt_length - 1]));
+  assert (elt_length > 0
+          && (IS_DIR_SEP_CH (elt[elt_length - 1])
+              || IS_DEVICE_SEP (elt[elt_length - 1])));
 
 #if defined (WIN32)
   strcpy(dirname, FN_STRING(name));
@@ -407,17 +409,44 @@ kpathsea_element_dirs (kpathsea kpse, string elt)
   str_llist_type *ret;
   unsigned i;
 
+#ifdef _WIN32
+  char *tname = NULL;
+  wchar_t *wtname = NULL;
+#endif /* _WIN32 */
+
   /* If given nothing, return nothing.  */
   if (!elt || !*elt)
     return NULL;
 
+#ifdef _WIN32
+/*
+  Change encoding of a variable into kpse->File_system_codepage
+  to support non-ascii values for the variable.
+*/
+  if (kpse->File_system_codepage != kpse->Win32_codepage) {
+    wtname = get_wstring_from_mbstring (kpse->Win32_codepage,
+                                        elt, wtname = NULL);
+    tname = get_mbstring_from_wstring (kpse->File_system_codepage,
+                                       wtname, tname = NULL);
+    elt = tname;
+    free(wtname);
+  }
+#endif /* _WIN32 */
+
   /* Normalize ELT before looking for a cached value.  */
   i = kpathsea_normalize_path (kpse, elt);
 
   /* If we've already cached the answer for ELT, return it.  */
   ret = cached (kpse, elt);
+#ifdef _WIN32
+  if (ret) {
+    if (tname) free (tname);
+    return ret;
+  }
+#else
   if (ret)
     return ret;
+#endif /* _WIN32 */
 
   /* We're going to have a real directory list to return.  */
   ret = XTALLOC1 (str_llist_type);
@@ -445,6 +474,10 @@ kpathsea_element_dirs (kpathsea kpse, string elt)
     }
 #endif /* KPSE_DEBUG */
 
+#ifdef _WIN32
+  if (tname) free (tname);
+#endif /* _WIN32 */
+
   return ret;
 }
 
