@@ -1,27 +1,30 @@
 #include <kpathsea/kpathsea.h>
 #include <ptexenc/ptexenc.h>
+#include "version.h"
 #include "makejvf.h"
 #include "uniblock.h"
+#include "usrtable.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 FILE *vfp,*afp=NULL;
-char *atfmname,*vtfmname,*afmname,*vfname,*kanatfm,*jistfm,*ucsqtfm;
-int kanatume=-1,chotai=0,baseshift=0,minute=0,useset3=0,hankana=0,fidzero=0;
+char *atfmname,*vtfmname,*afmname,*vfname,*kanatfm,*jistfm,*ucsqtfm,*usertable;
+int kanatume=-1,chotai=0,baseshift=0,minute=0,useset3=0,hankana=0,fidzero=0,enhanced=0;
+int pstfm_nt;
 long ucs=0;
 
 int main(int argc, char ** argv)
 {
-	int i,j,ib;
+	int i,j;
 	int c;
-	long ch;
+	long ch,ch_max;
 
 	kpse_set_program_name(argv[0], "makejvf");
 	set_enc_string("sjis", "euc");
 
-	while ((c = getopt (argc, argv, "k:K:Ca:b:mu:3J:U:Hi")) != -1)
+	while ((c = getopt (argc, argv, "k:K:Ca:b:mu:3J:U:Hiet:")) != -1)
 		switch (c) {
 
 
@@ -65,8 +68,10 @@ int main(int argc, char ** argv)
 				ucs = ENTRY_J;
 			else if (!strcmp(optarg, "ks"))
 				ucs = ENTRY_K;
+			else if (!strcmp(optarg, "custom"))
+				ucs = ENTRY_CUSTOM;
 			else {
-				fprintf(stderr,"Charset is not set\n");
+				fprintf(stderr,"[Warning] Charset is not set.\n");
 				ucs = ENTRY_NO;
 			}
 			break;
@@ -85,6 +90,12 @@ int main(int argc, char ** argv)
 		case 'i':
 			fidzero=1;
 			break;
+		case 'e':
+			enhanced=1;
+			break;
+		case 't':
+			usertable = xstrdup(optarg);
+			break;
 		default:
 			usage();
 			exit(0);
@@ -120,15 +131,49 @@ int main(int argc, char ** argv)
 
 	tfmget(atfmname);
 
-	maketfm(vtfmname);
+	if (usertable) {
+		get_usertable(usertable);
+	}
+	if (ucs!=ENTRY_CUSTOM && usertable_charset_max>0) {
+		fprintf(stderr,
+			"[Warning] Custom charset is defined in usertable\n"
+			"[Warning]   but it will be ignored.\n");
+	}
+	if (ucs==ENTRY_CUSTOM && usertable_charset_max<1) {
+		fprintf(stderr,"No custom charset definition in usertable.\n");
+		exit(101);
+	}
+
+	vfp = vfopen(vfname);
+
+	pstfm_nt=1; /* initialize */
+	if (ucs) {
+		if (ucs==ENTRY_CUSTOM) ch_max=usertable_charset[usertable_charset_max-1].max;
+		else if (useset3) ch_max=0x2FFFF;
+		else ch_max=0xFFFF;
+		for (ch=0;ch<=ch_max;ch++) {
+			if (search_cjk_entry(ch,ucs))
+				writevfu(ch,vfp);
+		}
+	} else {
+		for (i=0;i<94;i++)
+			for (j=0;j<94;j++)
+				writevf((0x21+i)*256+(0x21+j),vfp);
+	}
+
+	vfclose(vfp);
 
 	if (kanatfm) {
 		if (FILESTRCASEEQ(&kanatfm[strlen(kanatfm)-4], ".tfm")) {
 			kanatfm[strlen(kanatfm)-4] = '\0';
 		}
 		maketfm(kanatfm);
+		pstfm_nt=1; /* already done*/
 	}
 
+	maketfm(vtfmname);
+	pstfm_nt=1; /* already done*/
+
 	if (jistfm) {
 		if (FILESTRCASEEQ(&jistfm[strlen(jistfm)-4], ".tfm")) {
 			jistfm[strlen(jistfm)-4] = '\0';
@@ -143,34 +188,20 @@ int main(int argc, char ** argv)
 		maketfm(ucsqtfm);
 	}
 
-	vfp = vfopen(vfname);
-
-	if (ucs) {
-		ib=0;
-		for (i=0;i<(useset3*2+1);i++)
-			for (j=0;j<65536;j++) {
-				ch=i*65536+j;
-				if (search_cjk_entry(&ib,ch,ucs))
-					writevfu(ch,vfp);
-			}
-	} else {
-		for (i=0;i<94;i++)
-			for (j=0;j<94;j++)
-				writevf((0x21+i)*256+(0x21+j),vfp);
-	}
-
-	vfclose(vfp);
-
 	exit(0);
 }
 
 void usage(void)
 {
-	fputs2("MAKEJVF ver.1.1a-u1.22 -- make Japanese VF file.\n", stderr);
+	fprintf(stderr, "MAKEJVF version %s -- make Japanese VF from a JFM file.\n", VERSION);
+	fputs2("Usage:\n", stderr);
 	fputs2("%% makejvf [<options>] <TFMfile> <PSfontTFM>\n", stderr);
-	fputs2("options:\n", stderr);
+	fputs2("  <TFMfile>:   Name of input pTeX/upTeX JFM file.\n", stderr);
+	fputs2("               The basename is inherited by the name of output VF file.\n", stderr);
+	fputs2("  <PSfontTFM>: Name of output PSfont JFM file.\n", stderr);
+	fputs2("Options:\n", stderr);
 	fputs2("-C           ĹΥ⡼\n", stderr);
-	fputs2("-K <TFMfile> Ѥ˺PSեTFM̾\n", stderr);
+	fputs2("-K <PS-TFM>  Ѥ˺PSեTFM̾\n", stderr);
 	fputs2("-b <>    ١饤\n", stderr);
 	fputs2("             ʸι⤵1000Ȥǻ\n", stderr);
 	fputs2("             ץ饹ʸꡢޥʥʸ夬\n", stderr);
@@ -178,12 +209,18 @@ void usage(void)
 	fputs2("-a <AFMfile> AFMե̾ʤʵͤ˻ѡ\n", stderr);
 	fputs2("-k <>    ʵͤޡ\n", stderr);
 	fputs2("             ʸ1000Ȥǻꡣ-aץȶ˻\n", stderr);
+	fputs2("-i           Start mapped font ID from No. 0\n", stderr);
+	fputs2("-e           Enhanced mode; the horizontal shift amount is determined\n", stderr);
+	fputs2("             from the glue/kern table of <TFMfile> input\n", stderr);
+	fputs2("-t <CNFfile> Use <CNFfile> as a configuration file\n", stderr);
 	fputs2("-u <Charset> UCS mode\n", stderr);
 	fputs2("             <Charset> gb : GB,  cns : CNS,  ks : KS\n", stderr);
 	fputs2("                       jis : JIS,  jisq : JIS quote only\n", stderr);
-	fputs2("-J <TFMfile> JIS encoded PS font TFM name for quote, double quote (with UCS mode)\n", stderr);
-	fputs2("-U <TFMfile> UCS encoded PS font TFM name for quote, double quote (with UCS mode)\n", stderr);
-	fputs2("-3           use set3 (with UCS mode)\n", stderr);
-	fputs2("-H           use half-width katakana (with UCS mode)\n", stderr);
-	fputs2("-i           font ID from No.0\n", stderr);
+	fputs2("                       custom : Use user-defined CHARSET from <CNFfile>\n", stderr);
+	fputs2("Options below are effective only in UCS mode:\n", stderr);
+	fputs2("-J <PS-TFM>  Map single/double quote to another JIS-encoded PSfont TFM\n", stderr);
+	fputs2("-U <PS-TFM>  Map single/double quote to another UCS-encoded PSfont TFM\n", stderr);
+	fputs2("-3           Use set3, that is, enable non-BMP characters support\n", stderr);
+	fputs2("-H           Use half-width katakana\n", stderr);
+	fprintf(stderr, "Email bug reports to %s.\n", BUG_ADDRESS);
 }
