@@ -1,8 +1,7 @@
-
-
-extern char *vtfmname,*kanatfm,*jistfm,*ucsqtfm;
-extern int unit,zh,zw,jfm_id;
-extern int kanatume,chotai,baseshift,minute,hankana,fidzero;
+extern char *vtfmname,*kanatfm,*jistfm,*ucsqtfm,*usertable;
+extern int unit,zh,zw,jfm_id,rightamount;
+extern int kanatume,chotai,baseshift,minute,hankana,fidzero,enhanced;
+extern int pstfm_nt;
 extern FILE *afp;
 extern long ucs;
 
