@@ -19,7 +19,7 @@ char *styfile,*idxfile[256],*indfile,*dicfile,*logfile;
 #endif
 KpathseaSupportInfo kp_ist,kp_dict;
 
-#define VERSION "version 2.6f [14-Aug-2009]"
+#define VERSION "version 3.0 [15-May-2018]"
 
 int main(int argc, char **argv)
 {
@@ -31,11 +31,8 @@ int main(int argc, char **argv)
 
 #ifdef WIN32
 	_setmaxstdio(2048);
-        set_enc_string("sjis", "euc");
-        sjisterminal = 0;
-#else
-        set_enc_string(NULL, "euc");
 #endif
+	set_enc_string(NULL, "uptex");
 	kpse_set_program_name(argv[0], "mendex");
 
 	p = getenv ("PTEX_KANJI_ENC");
@@ -169,14 +166,8 @@ int main(int argc, char **argv)
 				set_enc_string("SJIS", NULL);
 				break;
 
-#ifdef WIN32
-			case 'T':
-				sjisterminal = 1;
-				break;
-#endif
-
 			case 'U':
-				set_enc_string("UTF8", "uptex");
+				set_enc_string("UTF8", NULL);
 				break;
 
 
@@ -222,9 +213,6 @@ int main(int argc, char **argv)
 				fprintf(stderr,"-E      EUC mode.\n");
 				fprintf(stderr,"-J      JIS mode.\n");
 				fprintf(stderr,"-S      ShiftJIS mode.\n");
-#ifdef WIN32
-				fprintf(stderr,"-T      ShiftJIS terminal.\n");
-#endif
 				fprintf(stderr,"-U      UTF-8 mode.\n");
 				fprintf(stderr,"-I enc  internal encoding for keywords (enc: euc or utf8).\n");
 				fprintf(stderr,"idx...  input files.\n");
