@@ -32,6 +32,9 @@
 #include <kpathsea/config.h>
 #include <kpathsea/c-fopen.h>
 #include <kpathsea/getopt.h>
+#if defined(WIN32)
+#include <kpathsea/variable.h>
+#endif
 #else
 #define FOPEN_RBIN_MODE  "rb"
 #define FOPEN_WBIN_MODE  "wb"
@@ -50,6 +53,11 @@ extern int optind;
 #include <stdio.h>
 #include <ctype.h>
 
+#if defined(WIN32) && defined(KPATHSEA)
+#undef fopen
+#define fopen  fsyscp_fopen
+#endif
+
 char *ProgName;
 
 /*
@@ -85,6 +93,9 @@ long	StartOfLastPage;	/* The file position just before we started
 long	CurrentPosition;	/* The current position of the file */
 
 int	NumberOfOutputPages;	/* number of pages in new DVI file */
+#ifdef ASCIIPTEX
+int	ptexdvi;		/* true => dvi format is extended (TATEKUMI) */
+#endif /* ASCIIPTEX */
 
 i32	Numerator;		/* numerator from current DVI file */
 i32	Denominator;		/* denominator from current DVI file */
@@ -225,6 +236,11 @@ WritePostAmble(void)
 
 	putbyte(outf, DVI_POSTPOST);
 	PutLong(outf, postpos);
+#ifdef ASCIIPTEX
+	if (ptexdvi)
+	  putbyte(outf, DVI_PTEXVERSION);
+	else
+#endif /* ASCIIPTEX */
 	putbyte(outf, DVI_VERSION);
 	putbyte(outf, DVI_FILLER);
 	putbyte(outf, DVI_FILLER);
@@ -372,6 +388,9 @@ doit(const char *name, FILE *fp)
 {
 	static int started;
 
+#ifdef	ASCIIPTEX
+	ptexdvi = 0;
+#endif
 	DVIFileName = name;
 	inf = fp;
 	if (HandlePreAmble(started ? 0 : 1))
@@ -388,6 +407,17 @@ main(int argc, char **argv)
 	register int c;
 	register char *s;
 	FILE *f;
+#if defined(WIN32) && defined(KPATHSEA)
+	int ac;
+	char **av, *enc;
+
+	kpse_set_program_name(argv[0], "dviconcat");
+	enc = kpse_var_value("command_line_encoding");
+	if (get_command_line_args_utf8(enc, &ac, &av)) {
+		argc = ac;
+		argv = av;
+	}
+#endif
 
 	ProgName = *argv;
 
@@ -664,7 +694,12 @@ char	oplen[128] = {
 	0,			/* DVI_PRE */
 	0,			/* DVI_POST */
 	0,			/* DVI_POSTPOST */
+#ifdef ASCIIPTEX
+	0, 0, 0, 0, 0,		/* 250 .. 254 */
+	0,			/* DVI_DIR */
+#else /* !ASCIIPTEX */
 	0, 0, 0, 0, 0, 0,	/* 250 .. 255 */
+#endif /* !ASCIIPTEX */
 };
 
 /*
@@ -751,6 +786,16 @@ HandleDVIFile(void)
 				HandleFontDef(p);
 				continue;
 
+#ifdef ASCIIPTEX
+			case DT_DIR:
+				ptexdvi = 1;
+
+				putbyte(outf, c);
+				putbyte(outf, p);
+				CurrentPosition += 2;
+				continue;
+#endif /* ASCIIPTEX */
+
 			default:
 				panic("HandleDVIFile DVI_DT(%d)=%d",
 				    c, DVI_DT(c));
