@@ -8,9 +8,16 @@
 --
 -- License: LPPL
 --
-local http = require("socket.http");
+local http = require("socket.http")
 local ltn12 = require("ltn12")
-local url = require("socket.url")
+-- local url = require("socket.url")
+-- avoid bug in luatex v1.0.7
+-- copied socket.url's escape function into local function url_escape
+function url_escape(s)
+    return (string.gsub(s, "([^A-Za-z0-9_])", function(c)
+        return string.format("%%%02x", string.byte(c))
+    end))
+end
 
 local OSMURL = "http://open.mapquestapi.com/staticmap/v4/getplacemap"
 local GMURL = "http://maps.googleapis.com/maps/api/staticmap"
@@ -38,9 +45,10 @@ local PITCH = ""
 local LANGUAGE = ""
 local GPFILE = ""
 local KMLFILE = ""
+local BOUND = 0.1
 local OFILE = "getmap"
 local QUIET = "false"
-local VERSION = "v1.4 (18/09/2014)"
+local VERSION = "v1.5a (2018/07/18)"
 
 function pversion()
   print("getmapdl.lua " .. VERSION)
@@ -130,6 +138,10 @@ getmapdl.lua [options]
 
  -K  specify the kml file
 
+ gps2gps mode only:
+
+ -B  specify the bound for reducing way points (default: 0.1)
+
 ]])
   pversion()
 end
@@ -185,6 +197,23 @@ function printepl(epltable)
   print(stingwithoutcontrolcharacters)
 end
 
+function isnotnumber(number)
+  if tonumber(number) == nil then
+    return true
+  else
+    return false
+  end
+end
+
+function dbtbound(Onum, num, bound)
+  local absdiff = math.abs(tonumber(Onum) - tonumber(num))
+  if absdiff >= tonumber(bound) then
+    return true
+  else
+    return false
+  end
+end
+
 do
   local newarg = {}
   local i, limit = 1, #arg
@@ -258,6 +287,9 @@ do
     elseif arg[i] == "-K" then
       KMLFILE = arg[i+1]
       i = i + 1
+    elseif arg[i] == "-B" then
+      BOUND = arg[i+1]
+      i = i + 1
     elseif arg[i] == "-o" then
       OFILE = arg[i+1]
       i = i + 1
@@ -368,6 +400,8 @@ if MODE == "gpx2gps" then
     if string.match(line, "<trkpt") then
       latitude = string.match(line, 'lat="(.-)"')
       longitude = string.match(line, 'lon="(.-)"')
+      latitude = round(latitude,5)
+      longitude = round(longitude,5)
       print(latitude .. "," .. longitude)
     end
     if string.match(line, "</trk>") then
@@ -377,26 +411,128 @@ if MODE == "gpx2gps" then
   os.exit(0)
 end
 
+if MODE == "gps2gps" then
+  local file = GPFILE
+  local bound = BOUND
+  local incount = 0
+  local outcount = 0
+  local routecount = 1
+  local latitude
+  local longitude
+  local Olatitude
+  local Olongitude
+  local Llatitude
+  local Llongitude
+  local ignorenextline = false
+  local firstroute = true
+
+  for line in io.lines(file)
+  do
+    latitude, longitude = line:match("([^,]+),([^,]+)")
+    if ignorenextline == true then
+      line = ""
+      ignorenextline = false
+    end
+    -- if line contains "Point:" then gps coordinates
+    -- in the next line must be ignored!
+    if line:match("Point:") then
+      ignorenextline = true
+    end
+    if line:match("Route:") then
+      if firstroute == true then
+        firstroute = false
+        routecount = 0
+      else
+        -- print last pair of coordinates
+        print(Llatitude .. "," .. Llongitude)
+        outcount = outcount + 1
+        io.stderr:write("\nRoute " .. routecount .. ": reduced gps coordinates (Bound = " .. bound .. "): " .. incount .. " -> " .. outcount)
+      end
+      incount = 0
+      outcount = 0
+      routecount = routecount + 1
+      print("\n" .. line .. "\n")
+      line = ""
+    end
+    if line == "" or isnotnumber(latitude) or isnotnumber(longitude)
+    then
+    -- empty line or no gps coordinates -> do nothing
+    else
+      latitude = round(latitude,5)
+      longitude = round(longitude,5)
+      Llatitude = latitude
+      Llongitude = longitude
+      incount = incount + 1
+      if incount == 1 then
+        Olatitude = latitude
+        Olongitude = longitude
+        print(Olatitude .. "," .. Olongitude)
+        outcount = outcount + 1
+      else
+        if dbtbound(Olatitude,latitude,bound) or dbtbound(Olongitude,longitude,bound) then
+          print(latitude .. "," .. longitude)
+          outcount = outcount + 1
+          Olatitude = latitude
+          Olongitude = longitude
+        end
+      end
+    end
+  end
+  -- print last pair of coordinates
+  print(Llatitude .. "," .. Llongitude)
+  outcount = outcount + 1
+  io.stderr:write("\nRoute " .. routecount .. ": reduced gps coordinates (Bound = " .. bound .. "): " .. incount .. " -> " .. outcount)
+  os.exit(0)
+end
+
 if MODE == "gps2epl" then
   local file = GPFILE
   local Olatitude = 0
   local Olongitude = 0
   local epl = {}
+  local firstroute = true
 
   for line in io.lines(file)
   do
     local latitude
     local longitude
     local encnum
+
     latitude, longitude = line:match("([^,]+),([^,]+)")
-    latitude = round(latitude,5)*100000
-    longitude = round(longitude,5)*100000
-    encnum = encodeNumber(latitude - Olatitude)
-    table.insert(epl,encnum)
-    encnum = encodeNumber(longitude - Olongitude)
-    table.insert(epl,encnum)
-    Olatitude = latitude
-    Olongitude = longitude
+    if ignorenextline == true then
+      line = ""
+      ignorenextline = false
+    end
+    -- if line contains "Point:" then gps coordinates
+    -- in the next line must be ignored!
+    if line:match("Point:") then
+      ignorenextline = true
+    end
+    if line:match("Route:") then
+      if firstroute == true then
+         firstroute = false
+      else
+        printepl(epl)
+        Olatitude = 0
+        Olongitude = 0
+        epl = {}
+      end
+      print("\n" .. line .. "\n")
+      line = ""
+    end
+    if line == "" or isnotnumber(latitude) or isnotnumber(longitude)
+    then
+    -- empty line or no gps coordinates -> do nothing
+    else
+      latitude = round(latitude,5)*100000
+      longitude = round(longitude,5)*100000
+      encnum = encodeNumber(latitude - Olatitude)
+      table.insert(epl,encnum)
+      encnum = encodeNumber(longitude - Olongitude)
+      table.insert(epl,encnum)
+      Olatitude = latitude
+      Olongitude = longitude
+    end
   end
   printepl(epl)
   os.exit(0)
@@ -452,11 +588,13 @@ if MODE == "kml2gps" or MODE == "kml2epl" then
         end
         for cocsv in string.gmatch(colist, "%S+") do
            longitude, latitude, altitude = cocsv:match("([^,]+),([^,]+),([^,]+)")
+           latitude = round(latitude,5)
+           longitude = round(longitude,5)
            if MODE == "kml2epl" then
              local encnum
              if cotype == "route" then
-               latitude = round(latitude,5)*100000
-               longitude = round(longitude,5)*100000
+               latitude = latitude*100000
+               longitude = longitude*100000
                encnum = encodeNumber(latitude - Olatitude)
                table.insert(epl,encnum)
                encnum = encodeNumber(longitude - Olongitude)
@@ -634,37 +772,37 @@ local UOFILE = ""
 local IMGURL = ""
 
 if MODE == "gm" then
-  ULOCATION = "center=" .. url.escape(LOCATION)
+  ULOCATION = "center=" .. url_escape(LOCATION)
   if MARKERS == "" then
-    UMARKERS = "&markers=size:mid|color:" .. COLOR .. "|label:" .. NUMBER .. "|" .. url.escape(LOCATION)
-    UZOOM = "&zoom=" .. url.escape(ZOOM)
+    UMARKERS = "&markers=size:mid|color:" .. COLOR .. "|label:" .. NUMBER .. "|" .. url_escape(LOCATION)
+    UZOOM = "&zoom=" .. url_escape(ZOOM)
   else
-    UMARKERS = "" .. url.escape(MARKERS)
+    UMARKERS = "" .. url_escape(MARKERS)
     -- correct cruft escaping of "&markers="
     UMARKERS = UMARKERS:gsub("%%26markers%%3d","&markers=")
     UZOMM = ""
     if CENTER == "" then
       ULOCATION = ""
     else
-      ULOCATION = "center=" .. url.escape(CENTER)
+      ULOCATION = "center=" .. url_escape(CENTER)
     end
   end
-  USIZE = "&size=" .. url.escape(SIZE)
-  USCALE = "&scale=" .. url.escape(SCALE)
-  UTYPE = "&maptype=" .. url.escape(TYPE)
-  UIMAGETYPE = "&format=" .. url.escape(IMAGETYPE)
+  USIZE = "&size=" .. url_escape(SIZE)
+  USCALE = "&scale=" .. url_escape(SCALE)
+  UTYPE = "&maptype=" .. url_escape(TYPE)
+  UIMAGETYPE = "&format=" .. url_escape(IMAGETYPE)
   if IMAGETYPE == "jpg-baseline" then
     IMAGETYPE = "jpg"
   end
   if VISIBLE == "" then
     UVISIBLE = ""
   else
-    UVISIBLE = "&visible=" .. url.escape(VISIBLE)
+    UVISIBLE = "&visible=" .. url_escape(VISIBLE)
   end
   if IPATH == "" then
     UIPATH = ""
   else
-    UIPATH = "" .. url.escape(IPATH)
+    UIPATH = "" .. url_escape(IPATH)
     -- correct cruft escaping of "&path="
     UIPATH = UIPATH:gsub("%%26path%%3d","&path=")
   end
@@ -674,7 +812,7 @@ if MODE == "gm" then
     EPLFILE = io.open(FPATH, "r")
     local contents = EPLFILE:read()
     EPLFILE:close()
-    UFPATH = "" .. url.escape(contents)
+    UFPATH = "" .. url_escape(contents)
     -- correct cruft escaping of "&path="
     UFPATH = UFPATH:gsub("%%26path%%3d","&path=")
   end
@@ -686,25 +824,25 @@ if MODE == "gm" then
   UOFILE = OFILE .. "." .. IMAGETYPE
   IMGURL = GMURL .. "?" .. ULOCATION .. USIZE .. UZOOM .. UMARKERS .. UTYPE .. USCALE .. UIMAGETYPE .. UVISIBLE .. UIPATH .. UFPATH .. ULANGUAGE .. "&sensor=false"
 elseif MODE == "gsv" then
-  ULOCATION = "location=" .. url.escape(LOCATION)
-  USIZE = "&size=" .. url.escape(SIZE)
-  UHEADING = "&heading=" .. url.escape(HEADING)
-  UFOV = "&fov=" .. url.escape(FOV)
-  UPITCH = "&pitch=" .. url.escape(PITCH)
+  ULOCATION = "location=" .. url_escape(LOCATION)
+  USIZE = "&size=" .. url_escape(SIZE)
+  UHEADING = "&heading=" .. url_escape(HEADING)
+  UFOV = "&fov=" .. url_escape(FOV)
+  UPITCH = "&pitch=" .. url_escape(PITCH)
   UOFILE = OFILE .. ".jpg"
   IMGURL = GSVURL .. "?" .. ULOCATION .. USIZE .. UHEADING .. UFOV .. UPITCH .. "&sensor=false"
 elseif MODE == "osm" then
-  UKEY = "?key=" .. url.escape(KEY)
-  ULOCATION = "&location=" .. url.escape(LOCATION)
-  USIZE = "&size=" .. url.escape(SIZE)
+  UKEY = "?key=" .. url_escape(KEY)
+  ULOCATION = "&location=" .. url_escape(LOCATION)
+  USIZE = "&size=" .. url_escape(SIZE)
   if ZOOM == "" then
-    USCALEZOOM = "&scale=" .. url.escape(SCALE)
+    USCALEZOOM = "&scale=" .. url_escape(SCALE)
   else
-    USCALEZOOM = "&zoom=" .. url.escape(ZOOM)
+    USCALEZOOM = "&zoom=" .. url_escape(ZOOM)
   end
-  UTYPE = "&type=" .. url.escape(TYPE)
-  UIMAGETYPE = "&imagetype=" .. url.escape(IMAGETYPE)
-  USHOWICON = "&showicon=" .. url.escape(COLOR) .. "-" .. url.escape(NUMBER)
+  UTYPE = "&type=" .. url_escape(TYPE)
+  UIMAGETYPE = "&imagetype=" .. url_escape(IMAGETYPE)
+  USHOWICON = "&showicon=" .. url_escape(COLOR) .. "-" .. url_escape(NUMBER)
   UOFILE = OFILE .. "." .. IMAGETYPE
   IMGURL = OSMURL .. UKEY .. ULOCATION .. USIZE .. USCALEZOOM .. UTYPE .. UIMAGETYPE .. USHOWICON
 end
@@ -717,7 +855,11 @@ if not ofile then
 end
 print("\n\ngetmapdl.lua:")
 print("url = " .. IMGURL)
+print("url length = " .. string.len(IMGURL) .. " bytes")
 print("output = " .. UOFILE)
+if string.len(IMGURL) > 2048 then
+  getmap_error(23, "URL exceeds length limit of 2048 bytes!")
+end
 ret, msg = http.request{
   url = IMGURL,
   sink = ltn12.sink.file(ofile)
