@@ -1,5 +1,5 @@
-/*
-Copyright 1996-2017 Han The Thanh, <thanh@pdftex.org>
+/* writefont.c: font descriptors and writing Type 1 fonts.
+Copyright 1996-2018 Han The Thanh, <thanh@pdftex.org>
 
 This file is part of pdfTeX.
 
@@ -484,16 +484,26 @@ static void write_fontdescriptor(fd_entry * fd)
     pdf_printf("/Flags %i\n", fd_flags);
     write_fontmetrics(fd);
     if (fd->ff_found) {
+#ifdef ENABLE_PDF_CHARSET /* just in case a builder wants it */
         if (is_subsetted(fd->fm) && is_type1(fd->fm)) {
-            /* /CharSet is optional; names may appear in any order */
+            /* CharSet is optional, but if it appears, it must be
+               correct. Unfortunately, there is no practical way we can
+               guarantee correctness with precomposed accent characters
+               in our usual fonts (EC, TX, etc.):
+              https://mailman.ntg.nl/pipermail/ntg-pdftex/2018-June/004251.html
+               Therefore, we disable its output. The code is left in
+               just in case it turns out that something important was
+               relying on the (incorrect) CharSet. */
             assert(fd->gl_tree != NULL);
             avl_t_init(&t, fd->gl_tree);
+            /* Names may appear in any order. */
             pdf_puts("/CharSet (");
             for (glyph = (char *) avl_t_first(&t, fd->gl_tree); glyph != NULL;
                  glyph = (char *) avl_t_next(&t))
                 pdf_printf("/%s", glyph);
             pdf_puts(")\n");
         }
+#endif /* ENABLE_PDF_CHARSET */
         if (is_type1(fd->fm))
             pdf_printf("/FontFile %i 0 R\n", (int) fd->ff_objnum);
         else if (is_truetype(fd->fm))
@@ -660,17 +670,45 @@ static void create_fontdictionary(fm_entry * fm, integer font_objnum,
         write_fontdictionary(fo);
 }
 
-/**********************************************************************/
 
-void dopdffont(integer font_objnum, internalfontnumber f)
+
+/* This is called font_has_subset in luatex, but it returns 1 if any
+   characters from the font are used, and 0 if not (using fontbc and
+   fontec as the endpoints to check), i.e., whether any characters are
+   actually used from the font. */
+
+static int
+font_is_used(internalfontnumber f)
+{
+    int i, s;
+    /* search for |first_char| and |last_char| */
+    for (i = fontbc[f]; i <= fontec[f]; i++)
+        if (pdfcharmarked(f, i))
+            break;
+    s = i;
+    for (i = fontec[f]; i >= fontbc[f]; i--)
+        if (pdfcharmarked(f, i))
+            break;
+    if (s > i)
+        return 0;
+    else
+        return 1;
+}
+
+void
+dopdffont(integer font_objnum, internalfontnumber f)
 {
     fm_entry *fm;
+    if (!font_is_used(f))
+        return; /* avoid failed assertion in create_fontdictionary,
+                mailman.ntg.nl/pipermail/ntg-pdftex/2018-January/004209.html */
+       
+
     fm = hasfmentry(f) ? (fm_entry *) pdffontmap[f] : NULL;
-    if (fm == NULL || (fm->ps_name == NULL && fm->ff_name == NULL))
-        writet3(font_objnum, f);
+    if (fm == NULL || is_pk(fm))
+        writet3(fm, font_objnum, f);
     else
         create_fontdictionary(fm, font_objnum, f);
 }
 
-/**********************************************************************/
 // vim: ts=4
